@@ -1,5 +1,4 @@
 //go:build ignore
-// +build ignore
 
 package main
 
@@ -12,7 +11,7 @@ import (
 )
 
 func main() {
-	fh, err := os.Create(os.ExpandEnv("./Documentation/cli/README.md"))
+	fh, err := os.Create("./Documentation/cli/README.md")
 	if err != nil {
 		log.Fatalf("could not create README.md: %v", err)
 	}
