@@ -1,5 +1,4 @@
 //go:build ignore
-// +build ignore
 
 package main
 
@@ -10,6 +9,7 @@ import (
 	"path/filepath"
 
 	"github.com/go-delve/delve/cmd/dlv/cmds"
+	"github.com/go-delve/delve/cmd/dlv/cmds/helphelpers"
 	"github.com/spf13/cobra/doc"
 )
 
@@ -21,12 +21,19 @@ func main() {
 		usageDir = os.Args[1]
 	}
 	root := cmds.New(true)
+
+	cmdnames := []string{}
+	for _, subcmd := range root.Commands() {
+		cmdnames = append(cmdnames, subcmd.Name())
+	}
+	helphelpers.Prepare(root)
 	doc.GenMarkdownTree(root, usageDir)
+	root = nil
 	// GenMarkdownTree ignores additional help topic commands, so we have to do this manually
-	for _, cmd := range root.Commands() {
-		if cmd.Run == nil {
-			doc.GenMarkdownTree(cmd, usageDir)
-		}
+	for _, cmdname := range cmdnames {
+		cmd, _, _ := cmds.New(true).Find([]string{cmdname})
+		helphelpers.Prepare(cmd)
+		doc.GenMarkdownTree(cmd, usageDir)
 	}
 	fh, err := os.OpenFile(filepath.Join(usageDir, "dlv.md"), os.O_APPEND|os.O_WRONLY, 0)
 	if err != nil {
