@@ -2,6 +2,7 @@ package main
 
 import (
 	"fmt"
+	"io"
 	"log"
 	"os"
 	"os/exec"
@@ -54,7 +55,7 @@ func NewMakeCommands() *cobra.Command {
 			} else {
 				execute("go", "build", "-ldflags", "-extldflags -static", tagFlags(), buildFlags(), DelveMainPackagePath)
 			}
-			if runtime.GOOS == "darwin" && os.Getenv("CERT") != "" && canMacnative() {
+			if runtime.GOOS == "darwin" && os.Getenv("CERT") != "" && canMacnative() && !isCodesigned("./dlv") {
 				codesign("./dlv")
 			}
 		},
@@ -70,7 +71,7 @@ func NewMakeCommands() *cobra.Command {
 		Short: "Installs delve",
 		Run: func(cmd *cobra.Command, args []string) {
 			execute("go", "install", tagFlags(), buildFlags(), DelveMainPackagePath)
-			if runtime.GOOS == "darwin" && os.Getenv("CERT") != "" && canMacnative() {
+			if runtime.GOOS == "darwin" && os.Getenv("CERT") != "" && canMacnative() && !isCodesigned(installedExecutablePath()) {
 				codesign(installedExecutablePath())
 			}
 		},
@@ -130,7 +131,7 @@ This option can only be specified if testset is basic or a single package.`)
 
 func checkCert() bool {
 	// If we're on OSX make sure the proper CERT env var is set.
-	if os.Getenv("TRAVIS") == "true" || runtime.GOOS != "darwin" || os.Getenv("CERT") != "" {
+	if runtime.GOOS != "darwin" || os.Getenv("CERT") != "" {
 		return true
 	}
 
@@ -225,6 +226,15 @@ func getoutput(cmd string, args ...interface{}) string {
 	return string(out)
 }
 
+func isCodesigned(path string) bool {
+	x := exec.Command("codesign", "--verify", path)
+	x.Stdout = io.Discard
+	x.Stderr = io.Discard
+	x.Env = os.Environ()
+	err := x.Run()
+	return err == nil && x.ProcessState != nil && x.ProcessState.Success()
+}
+
 func codesign(path string) {
 	execute("codesign", "-s", os.Getenv("CERT"), path)
 }
@@ -291,6 +301,9 @@ func tagFlags() string {
 	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
 		tags = append(tags, "exp.winarm64")
 	}
+	if runtime.GOOS == "linux" && runtime.GOARCH == "ppc64le" {
+		tags = append(tags, "exp.linuxppc64le")
+	}
 	if Tags != nil && len(*Tags) > 0 {
 		tags = append(tags, *Tags...)
 	}
@@ -325,9 +338,6 @@ func testFlags() []string {
 	}
 	if NOTimeout {
 		testFlags = append(testFlags, "-timeout", "0")
-	} else if os.Getenv("TRAVIS") == "true" {
-		// Make test timeout shorter than Travis' own timeout so that Go can report which test hangs.
-		testFlags = append(testFlags, "-timeout", "9m")
 	}
 	if len(os.Getenv("TEAMCITY_VERSION")) > 0 {
 		testFlags = append(testFlags, "-json")
@@ -341,20 +351,6 @@ func testFlags() []string {
 func testCmd(cmd *cobra.Command, args []string) {
 	checkCertCmd(nil, nil)
 
-	if os.Getenv("TRAVIS") == "true" && runtime.GOOS == "darwin" {
-		fmt.Println("Building with native backend")
-		execute("go", "build", "-tags=macnative", buildFlags(), DelveMainPackagePath)
-
-		fmt.Println("\nBuilding without native backend")
-		execute("go", "build", buildFlags(), DelveMainPackagePath)
-
-		fmt.Println("\nTesting")
-		os.Setenv("PROCTEST", "lldb")
-		env := []string{}
-		executeq(env, "sudo", "-E", "go", "test", testFlags(), allPackages())
-		return
-	}
-
 	if TestSet == "" && TestBackend == "" && TestBuildMode == "" {
 		if TestRegex != "" {
 			fmt.Printf("Can not use --test-run without --test-set\n")
@@ -395,7 +391,9 @@ func testStandard() {
 		dopie := false
 		switch runtime.GOOS {
 		case "linux":
-			dopie = true
+			if runtime.GOARCH != "ppc64le" {
+				dopie = true
+			}
 		case "windows":
 			// windows/arm64 always uses pie buildmode, no need to test everything again.
 			// only on Go 1.15 or later, with CGO_ENABLED and gcc found in path
@@ -410,6 +408,11 @@ func testStandard() {
 					}
 				}
 			}
+		case "darwin":
+			if runtime.GOARCH == "amd64" {
+				// arm64 can only build in pie mode
+				dopie = true
+			}
 		}
 		if dopie {
 			fmt.Println("\nTesting PIE buildmode, default backend")
