@@ -9,7 +9,6 @@ import (
 	"go/token"
 	"go/types"
 	"io"
-	"io/ioutil"
 	"net"
 	"os"
 	"os/exec"
@@ -188,10 +187,10 @@ func testOutput(t *testing.T, dlvbin, output string, delveCmds []string) (stdout
 	if err == nil {
 		// Sometimes delve on Windows can't remove the built binary before
 		// exiting and gets an "Access is denied" error when trying.
-		// See: https://travis-ci.com/go-delve/delve/jobs/296325131)
+		// See: https://travis-ci.com/go-delve/delve/jobs/296325131.
 		// We have added a delay to gobuild.Remove, but to avoid any test
 		// flakiness, we guard against this failure here as well.
-		if runtime.GOOS != "windows" || !strings.Contains(err.Error(), "Access is denied") {
+		if runtime.GOOS != "windows" {
 			t.Errorf("running %q: file %v was not deleted\nstdout is %q, stderr is %q", delveCmds, debugbin, stdout, stderr)
 		}
 		return
@@ -205,14 +204,17 @@ func testOutput(t *testing.T, dlvbin, output string, delveCmds []string) (stdout
 
 func getDlvBin(t *testing.T) string {
 	// In case this was set in the environment
-	// from getDlvBinEBPF lets clear it here so
+	// from getDlvBinEBPF lets clear it here, so
 	// we can ensure we don't get build errors
 	// depending on the test ordering.
-	os.Setenv("CGO_LDFLAGS", ldFlags)
+	t.Setenv("CGO_LDFLAGS", ldFlags)
 	var tags string
 	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
 		tags = "-tags=exp.winarm64"
 	}
+	if runtime.GOOS == "linux" && runtime.GOARCH == "ppc64le" {
+		tags = "-tags=exp.linuxppc64le"
+	}
 	return getDlvBinInternal(t, tags)
 }
 
@@ -306,7 +308,7 @@ func TestRedirect(t *testing.T) {
 
 	// and detach from and kill the headless instance
 	client := rpc2.NewClient(listenAddr)
-	_ = client.Detach(true)
+	client.Detach(true)
 	cmd.Wait()
 }
 
@@ -338,7 +340,7 @@ func checkAutogenDoc(t *testing.T, filename, gencommand string, generated []byte
 }
 
 func slurpFile(t *testing.T, filename string) []byte {
-	saved, err := ioutil.ReadFile(filename)
+	saved, err := os.ReadFile(filename)
 	if err != nil {
 		t.Fatalf("Could not read %s: %v", filename, err)
 	}
@@ -364,13 +366,14 @@ func diffMaybe(t *testing.T, filename string, generated []byte) {
 // TestGeneratedDoc tests that the autogenerated documentation has been
 // updated.
 func TestGeneratedDoc(t *testing.T) {
-	if strings.ToLower(os.Getenv("TRAVIS")) == "true" && runtime.GOOS == "windows" {
-		t.Skip("skipping test on Windows in CI")
-	}
 	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
 		//TODO(qmuntal): investigate further when the Windows ARM64 backend is more stable.
 		t.Skip("skipping test on Windows in CI")
 	}
+	if runtime.GOOS == "linux" && runtime.GOARCH == "ppc64le" {
+		//TODO(alexsaezm): finish CI integration
+		t.Skip("skipping test on Linux/PPC64LE in CI")
+	}
 	// Checks gen-cli-docs.go
 	var generatedBuf bytes.Buffer
 	commands := terminal.DebugCommands(nil)
@@ -383,7 +386,7 @@ func TestGeneratedDoc(t *testing.T) {
 	cmd.Dir = projectRoot()
 	err := cmd.Run()
 	assertNoError(err, t, "go run _scripts/gen-usage-docs.go")
-	entries, err := ioutil.ReadDir(tempDir)
+	entries, err := os.ReadDir(tempDir)
 	assertNoError(err, t, "ReadDir")
 	for _, doc := range entries {
 		docFilename := "Documentation/usage/" + doc.Name()
@@ -421,7 +424,7 @@ func TestExitInInit(t *testing.T) {
 	cmd.Dir = buildtestdir
 	out, err := cmd.CombinedOutput()
 	t.Logf("%q %v\n", string(out), err)
-	// dlv will exit anyway because stdin is not a tty but it will print the
+	// dlv will exit anyway because stdin is not a tty, but it will print the
 	// prompt once if the init file didn't call exit successfully.
 	if strings.Contains(string(out), "(dlv)") {
 		t.Fatal("init did not cause dlv to exit")
@@ -905,7 +908,7 @@ func TestTrace(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
 	if !bytes.Contains(output, expected) {
@@ -929,7 +932,7 @@ func TestTrace2(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
 	if !bytes.Contains(output, expected) {
@@ -941,7 +944,7 @@ func TestTrace2(t *testing.T) {
 func TestTraceMultipleGoroutines(t *testing.T) {
 	dlvbin := getDlvBin(t)
 
-	// TODO(derekparker) this test has to be a bit vague to avoid flakyness.
+	// TODO(derekparker) this test has to be a bit vague to avoid flakiness.
 	// I think a future improvement could be to use regexp captures to match the
 	// goroutine IDs at function entry and exit.
 	expected := []byte("main.callme(0, \"five\")\n")
@@ -957,7 +960,7 @@ func TestTraceMultipleGoroutines(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
 	if !bytes.Contains(output, expected) {
@@ -971,7 +974,7 @@ func TestTraceMultipleGoroutines(t *testing.T) {
 
 func TestTracePid(t *testing.T) {
 	if runtime.GOOS == "linux" {
-		bs, _ := ioutil.ReadFile("/proc/sys/kernel/yama/ptrace_scope")
+		bs, _ := os.ReadFile("/proc/sys/kernel/yama/ptrace_scope")
 		if bs == nil || strings.TrimSpace(string(bs)) != "0" {
 			t.Logf("can not run TestAttachDetach: %v\n", bs)
 			return
@@ -988,7 +991,7 @@ func TestTracePid(t *testing.T) {
 	assertNoError(targetCmd.Start(), t, "execute issue2023")
 
 	if targetCmd.Process == nil || targetCmd.Process.Pid == 0 {
-		t.Fatal("expected target process runninng")
+		t.Fatal("expected target process running")
 	}
 	defer targetCmd.Process.Kill()
 
@@ -1000,7 +1003,7 @@ func TestTracePid(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
 	if !bytes.Contains(output, expected) {
@@ -1028,7 +1031,7 @@ func TestTraceBreakpointExists(t *testing.T) {
 
 	defer cmd.Wait()
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
 	if bytes.Contains(output, []byte("Breakpoint exists")) {
@@ -1050,7 +1053,7 @@ func TestTracePrintStack(t *testing.T) {
 
 	defer cmd.Wait()
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
 	if !bytes.Contains(output, []byte("Stack:")) && !bytes.Contains(output, []byte("main.main")) {
@@ -1088,13 +1091,13 @@ func TestTraceEBPF(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
+	cmd.Wait()
 	if !bytes.Contains(output, expected) {
 		t.Fatalf("expected:\n%s\ngot:\n%s", string(expected), string(output))
 	}
-	cmd.Wait()
 }
 
 func TestTraceEBPF2(t *testing.T) {
@@ -1148,13 +1151,13 @@ func TestTraceEBPF2(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
+	cmd.Wait()
 	if !bytes.Contains(output, expected) {
 		t.Fatalf("expected:\n%s\ngot:\n%s", string(expected), string(output))
 	}
-	cmd.Wait()
 }
 
 func TestTraceEBPF3(t *testing.T) {
@@ -1196,13 +1199,61 @@ func TestTraceEBPF3(t *testing.T) {
 
 	assertNoError(cmd.Start(), t, "running trace")
 
-	output, err := ioutil.ReadAll(rdr)
+	output, err := io.ReadAll(rdr)
 	assertNoError(err, t, "ReadAll")
 
+	cmd.Wait()
 	if !bytes.Contains(output, expected) {
 		t.Fatalf("expected:\n%s\ngot:\n%s", string(expected), string(output))
 	}
+}
+
+func TestTraceEBPF4(t *testing.T) {
+	if os.Getenv("CI") == "true" {
+		t.Skip("cannot run test in CI, requires kernel compiled with btf support")
+	}
+	if runtime.GOOS != "linux" || runtime.GOARCH != "amd64" {
+		t.Skip("not implemented on non linux/amd64 systems")
+	}
+	if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 16) {
+		t.Skip("requires at least Go 1.16 to run test")
+	}
+	usr, err := user.Current()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if usr.Uid != "0" {
+		t.Skip("test must be run as root")
+	}
+
+	dlvbin := getDlvBinEBPF(t)
+
+	expected := []byte(`> (1) main.tracedFunction(0, true, 97)
+> (1) main.tracedFunction(1, false, 98)
+> (1) main.tracedFunction(2, false, 99)
+> (1) main.tracedFunction(3, false, 100)
+> (1) main.tracedFunction(4, false, 101)
+> (1) main.tracedFunction(5, true, 102)
+> (1) main.tracedFunction(6, false, 103)
+> (1) main.tracedFunction(7, false, 104)
+> (1) main.tracedFunction(8, false, 105)
+> (1) main.tracedFunction(9, false, 106)`)
+
+	fixtures := protest.FindFixturesDir()
+	cmd := exec.Command(dlvbin, "trace", "--ebpf", "--output", filepath.Join(t.TempDir(), "__debug"), filepath.Join(fixtures, "ebpf_trace3.go"), "main.traced")
+	rdr, err := cmd.StderrPipe()
+	assertNoError(err, t, "stderr pipe")
+	defer rdr.Close()
+
+	assertNoError(cmd.Start(), t, "running trace")
+
+	output, err := io.ReadAll(rdr)
+	assertNoError(err, t, "ReadAll")
+
 	cmd.Wait()
+	if !bytes.Contains(output, expected) {
+		t.Fatalf("expected:\n%s\ngot:\n%s", string(expected), string(output))
+	}
 }
 
 func TestDlvTestChdir(t *testing.T) {
@@ -1308,3 +1359,65 @@ func TestDefaultBinary(t *testing.T) {
 		t.Errorf("outputs match")
 	}
 }
+
+func TestUnixDomainSocket(t *testing.T) {
+	tmpdir := os.TempDir()
+	if tmpdir == "" {
+		return
+	}
+
+	listenPath := filepath.Join(tmpdir, "delve_test")
+
+	var err error
+
+	cmd := exec.Command("go", "run", "_scripts/make.go", "build")
+	cmd.Dir = projectRoot()
+	out, err := cmd.CombinedOutput()
+	if err != nil {
+		t.Fatalf("makefile error: %v\noutput %s\n", err, string(out))
+	}
+
+	dlvbin := filepath.Join(cmd.Dir, "dlv")
+	defer os.Remove(dlvbin)
+
+	fixtures := protest.FindFixturesDir()
+
+	buildtestdir := filepath.Join(fixtures, "buildtest")
+
+	cmd = exec.Command(dlvbin, "debug", "--headless=true", "--listen=unix:"+listenPath, "--api-version=2", "--backend="+testBackend, "--log", "--log-output=debugger,rpc")
+	cmd.Dir = buildtestdir
+	stderr, err := cmd.StderrPipe()
+	assertNoError(err, t, "stderr pipe")
+	defer stderr.Close()
+
+	assertNoError(cmd.Start(), t, "dlv debug")
+
+	scan := bufio.NewScanner(stderr)
+	// wait for the debugger to start
+	for scan.Scan() {
+		text := scan.Text()
+		t.Log(text)
+		if strings.Contains(text, "API server pid = ") {
+			break
+		}
+	}
+	go func() {
+		for scan.Scan() {
+			t.Log(scan.Text())
+			// keep pipe empty
+		}
+	}()
+
+	conn, err := net.Dial("unix", listenPath)
+	assertNoError(err, t, "dialing")
+
+	client := rpc2.NewClientFromConn(conn)
+	state := <-client.Continue()
+
+	if !state.Exited {
+		t.Fatal("Program did not exit")
+	}
+
+	client.Detach(true)
+	cmd.Wait()
+}
