@@ -101,7 +101,7 @@ func (b *Builder) TagOpen(tag dwarf.Tag, name string) dwarf.Offset {
 
 // SetHasChildren sets the current DIE as having children (even if none are added).
 func (b *Builder) SetHasChildren() {
-	if len(b.tagStack) <= 0 {
+	if len(b.tagStack) == 0 {
 		panic("NoChildren with no open tags")
 	}
 	b.tagStack[len(b.tagStack)-1].children = true
@@ -109,7 +109,7 @@ func (b *Builder) SetHasChildren() {
 
 // TagClose closes the current DIE.
 func (b *Builder) TagClose() {
-	if len(b.tagStack) <= 0 {
+	if len(b.tagStack) == 0 {
 		panic("TagClose with no open tags")
 	}
 	tag := b.tagStack[len(b.tagStack)-1]
@@ -167,8 +167,8 @@ func (b *Builder) Attr(attr dwarf.Attr, val interface{}) dwarf.Offset {
 		binary.Write(&b.loc, binary.LittleEndian, uint64(0))
 
 		for _, locentry := range x {
-			binary.Write(&b.loc, binary.LittleEndian, uint64(locentry.Lowpc))
-			binary.Write(&b.loc, binary.LittleEndian, uint64(locentry.Highpc))
+			binary.Write(&b.loc, binary.LittleEndian, locentry.Lowpc)
+			binary.Write(&b.loc, binary.LittleEndian, locentry.Highpc)
 			binary.Write(&b.loc, binary.LittleEndian, uint16(len(locentry.Loc)))
 			b.loc.Write(locentry.Loc)
 		}
