@@ -11,24 +11,34 @@ import (
 // entry specified by DW_AT_abstract_origin will be searched recursively.
 type Entry interface {
 	Val(dwarf.Attr) interface{}
+	AttrField(dwarf.Attr) *dwarf.Field
 }
 
 type compositeEntry []*dwarf.Entry
 
 func (ce compositeEntry) Val(attr dwarf.Attr) interface{} {
+	if f := ce.AttrField(attr); f != nil {
+		return f.Val
+	}
+	return nil
+}
+
+func (ce compositeEntry) AttrField(a dwarf.Attr) *dwarf.Field {
 	for _, e := range ce {
-		if r := e.Val(attr); r != nil {
-			return r
+		if f := e.AttrField(a); f != nil {
+			return f
 		}
 	}
 	return nil
 }
 
-// LoadAbstractOrigin loads the entry corresponding to the
-// DW_AT_abstract_origin of entry and returns a combination of entry and its
-// abstract origin.
-func LoadAbstractOrigin(entry *dwarf.Entry, aordr *dwarf.Reader) (Entry, dwarf.Offset) {
-	ao, ok := entry.Val(dwarf.AttrAbstractOrigin).(dwarf.Offset)
+// LoadAbstractOriginAndSpecification loads the entry corresponding to the
+// DW_AT_abstract_origin and/or DW_AT_specification of entry and returns a
+// combination of entry and its abstract origin. If a DIE has both a
+// specification and an abstract origin the specification will be ignored, the
+// DWARF standard is unclear on how this should be handled
+func LoadAbstractOriginAndSpecification(entry *dwarf.Entry, aordr *dwarf.Reader) (Entry, dwarf.Offset) {
+	ao, ok := getAbstractOriginOrSpecification(entry)
 	if !ok {
 		return entry, entry.Offset
 	}
@@ -43,7 +53,7 @@ func LoadAbstractOrigin(entry *dwarf.Entry, aordr *dwarf.Reader) (Entry, dwarf.O
 		}
 		r = append(r, e)
 
-		ao, ok = e.Val(dwarf.AttrAbstractOrigin).(dwarf.Offset)
+		ao, ok = getAbstractOriginOrSpecification(e)
 		if !ok {
 			break
 		}
@@ -52,6 +62,18 @@ func LoadAbstractOrigin(entry *dwarf.Entry, aordr *dwarf.Reader) (Entry, dwarf.O
 	return compositeEntry(r), entry.Offset
 }
 
+func getAbstractOriginOrSpecification(e *dwarf.Entry) (dwarf.Offset, bool) {
+	ao, ok := e.Val(dwarf.AttrAbstractOrigin).(dwarf.Offset)
+	if ok {
+		return ao, true
+	}
+	sp, ok := e.Val(dwarf.AttrSpecification).(dwarf.Offset)
+	if ok {
+		return sp, true
+	}
+	return dwarf.Offset(0), false
+}
+
 // Tree represents a tree of dwarf objects.
 type Tree struct {
 	Entry
@@ -85,7 +107,7 @@ func LoadTree(off dwarf.Offset, dw *dwarf.Data, staticBase uint64) (*Tree, error
 	if err != nil {
 		return nil, err
 	}
-	r.resolveAbstractEntries(rdr)
+	r.resolveAbstractAndSpecificationEntries(rdr)
 
 	return r, nil
 }
@@ -193,8 +215,10 @@ func max(a, b uint64) uint64 {
 }
 
 // fuseRanges fuses rngs2 into rngs1, it's the equivalent of
-//     normalizeRanges(append(rngs1, rngs2))
-// but more efficent.
+//
+//	normalizeRanges(append(rngs1, rngs2))
+//
+// but more efficient.
 func fuseRanges(rngs1, rngs2 [][2]uint64) [][2]uint64 {
 	if rangesContains(rngs1, rngs2) {
 		return rngs1
@@ -226,10 +250,10 @@ func rangeContains(a, b [2]uint64) bool {
 	return a[0] <= b[0] && a[1] >= b[1]
 }
 
-func (n *Tree) resolveAbstractEntries(rdr *dwarf.Reader) {
-	n.Entry, n.Offset = LoadAbstractOrigin(n.Entry.(*dwarf.Entry), rdr)
+func (n *Tree) resolveAbstractAndSpecificationEntries(rdr *dwarf.Reader) {
+	n.Entry, n.Offset = LoadAbstractOriginAndSpecification(n.Entry.(*dwarf.Entry), rdr)
 	for _, child := range n.Children {
-		child.resolveAbstractEntries(rdr)
+		child.resolveAbstractAndSpecificationEntries(rdr)
 	}
 }
 
