@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"debug/dwarf"
 	"debug/elf"
-	"debug/gosym"
 	"debug/macho"
 	"debug/pe"
 	"encoding/binary"
@@ -15,7 +14,6 @@ import (
 	"go/token"
 	"hash/crc32"
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"sort"
@@ -32,6 +30,7 @@ import (
 	"github.com/go-delve/delve/pkg/dwarf/op"
 	"github.com/go-delve/delve/pkg/dwarf/reader"
 	"github.com/go-delve/delve/pkg/goversion"
+	"github.com/go-delve/delve/pkg/internal/gosym"
 	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc/debuginfod"
 	"github.com/hashicorp/golang-lru/simplelru"
@@ -129,6 +128,7 @@ var (
 		elf.EM_X86_64:  true,
 		elf.EM_AARCH64: true,
 		elf.EM_386:     true,
+		elf.EM_PPC64:   true,
 	}
 
 	supportedWindowsArch = map[_PEMachine]bool{
@@ -279,7 +279,7 @@ func FindFileLocation(p Process, filename string, lineno int) ([]uint64, error)
 	return selectedPCs, nil
 }
 
-// inlRnage is the range of an inlined call
+// inlRange is the range of an inlined call
 type inlRange struct {
 	off   dwarf.Offset
 	depth uint32
@@ -687,6 +687,8 @@ func NewBinaryInfo(goos, goarch string) *BinaryInfo {
 		r.Arch = AMD64Arch(goos)
 	case "arm64":
 		r.Arch = ARM64Arch(goos)
+	case "ppc64le":
+		r.Arch = PPC64LEArch(goos)
 	}
 	return r
 }
@@ -854,8 +856,8 @@ type Image struct {
 
 	compileUnits []*compileUnit // compileUnits is sorted by increasing DWARF offset
 
-	dwarfTreeCache      *simplelru.LRU
-	runtimeMallocgcTree *godwarf.Tree // patched version of runtime.mallocgc's DIE
+	dwarfTreeCache  *simplelru.LRU
+	workaroundCache map[dwarf.Offset]*godwarf.Tree
 
 	// runtimeTypeToDIE maps between the offset of a runtime._type in
 	// runtime.moduledata.types and the offset of the DIE in debug_info. This
@@ -911,7 +913,7 @@ func (bi *BinaryInfo) AddImage(path string, addr uint64) error {
 
 // moduleDataToImage finds the image corresponding to the given module data object.
 func (bi *BinaryInfo) moduleDataToImage(md *moduleData) *Image {
-	fn := bi.PCToFunc(uint64(md.text))
+	fn := bi.PCToFunc(md.text)
 	if fn != nil {
 		return bi.funcToImage(fn)
 	}
@@ -1009,8 +1011,8 @@ func (image *Image) LoadError() error {
 }
 
 func (image *Image) getDwarfTree(off dwarf.Offset) (*godwarf.Tree, error) {
-	if image.runtimeMallocgcTree != nil && off == image.runtimeMallocgcTree.Offset {
-		return image.runtimeMallocgcTree, nil
+	if image.workaroundCache[off] != nil {
+		return image.workaroundCache[off], nil
 	}
 	if r, ok := image.dwarfTreeCache.Get(off); ok {
 		return r.(*godwarf.Tree), nil
@@ -1349,7 +1351,7 @@ func (bi *BinaryInfo) openSeparateDebugInfo(image *Image, exe *elf.File, debugIn
 
 		if debugFilePath != "" {
 			// CRC check
-			buf, err := ioutil.ReadFile(debugFilePath)
+			buf, err := os.ReadFile(debugFilePath)
 			if err == nil {
 				computedCRC := crc32.ChecksumIEEE(buf)
 				if crc != computedCRC {
@@ -1453,20 +1455,63 @@ func loadBinaryInfoElf(bi *BinaryInfo, image *Image, path string, addr uint64, w
 			if len(bi.Images) <= 1 {
 				fmt.Fprintln(os.Stderr, "Warning: no debug info found, some functionality will be missing such as stack traces and variable evaluation.")
 			}
-			symTable, err := readPcLnTableElf(elfFile, path)
+			cu := &compileUnit{}
+			cu.image = image
+			symTable, symTabAddr, err := readPcLnTableElf(elfFile, path)
 			if err != nil {
 				return fmt.Errorf("could not read debug info (%v) and could not read go symbol table (%v)", dwerr, err)
 			}
 			image.symTable = symTable
+			noPtrSectionData, err := elfFile.Section(".noptrdata").Data()
+			if err != nil {
+				return err
+			}
+			md, err := parseModuleData(noPtrSectionData, symTabAddr)
+			if err != nil {
+				return err
+			}
+			roDataAddr := elfFile.Section(".rodata").Addr
+			goFuncVal, err := findGoFuncVal(md, roDataAddr, bi.Arch.ptrSize)
+			if err != nil {
+				return err
+			}
+			prog := gosym.ProgContaining(elfFile, goFuncVal)
+			inlFuncs := make(map[string]*Function)
 			for _, f := range image.symTable.Funcs {
-				cu := &compileUnit{}
-				cu.image = image
-				fn := Function{Name: f.Name, Entry: f.Entry + image.StaticBase, End: f.End + image.StaticBase, cu: cu}
+				fnEntry := f.Entry + image.StaticBase
+				if prog != nil {
+					inlCalls, err := image.symTable.GetInlineTree(&f, goFuncVal, prog.Vaddr, prog.ReaderAt)
+					if err != nil {
+						return err
+					}
+					for _, inlfn := range inlCalls {
+						newInlinedCall := InlinedCall{cu: cu, LowPC: fnEntry + uint64(inlfn.ParentPC)}
+						if fn, ok := inlFuncs[inlfn.Name]; ok {
+							fn.InlinedCalls = append(fn.InlinedCalls, newInlinedCall)
+							continue
+						}
+						inlFuncs[inlfn.Name] = &Function{
+							Name:  inlfn.Name,
+							Entry: 0, End: 0,
+							cu: cu,
+							InlinedCalls: []InlinedCall{
+								newInlinedCall,
+							},
+						}
+					}
+				}
+				fn := Function{Name: f.Name, Entry: fnEntry, End: f.End + image.StaticBase, cu: cu}
 				bi.Functions = append(bi.Functions, fn)
 			}
+			for i := range inlFuncs {
+				bi.Functions = append(bi.Functions, *inlFuncs[i])
+			}
+			sort.Sort(functionsDebugInfoByEntry(bi.Functions))
 			for f := range image.symTable.Files {
 				bi.Sources = append(bi.Sources, f)
 			}
+			sort.Strings(bi.Sources)
+			bi.Sources = uniq(bi.Sources)
 			return nil
 		}
 		image.sepDebugCloser = sepFile
@@ -1508,6 +1553,53 @@ func loadBinaryInfoElf(bi *BinaryInfo, image *Image, path string, addr uint64, w
 	return nil
 }
 
+func findGoFuncVal(moduleData []byte, roDataAddr uint64, ptrsize int) (uint64, error) {
+	buf := new(bytes.Buffer)
+	err := binary.Write(buf, binary.LittleEndian, &roDataAddr)
+	if err != nil {
+		return 0, err
+	}
+	// Here we search for the value of `go.func.*` by searching through the raw bytes of the
+	// runtime.moduledata structure. Since we don't know the value that we are looking for,
+	// we use a known value, in this case the address of the .rodata section.
+	// This is because in the layout of the struct, the rodata member is right next to
+	// the value we need, making the math trivial once we find that member.
+	// We use `bytes.LastIndex` specifically because the `types` struct member can also
+	// contain the address of the .rodata section, so this pointer can appear multiple times
+	// in the raw bytes.
+	// Yes, this is very ill-advised low-level hackery but it works fine until
+	// https://github.com/golang/go/issues/58474#issuecomment-1785681472 happens.
+	// This code path also only runs in stripped binaries, so the whole implementation is
+	// best effort anyways.
+	rodata := bytes.LastIndex(moduleData, buf.Bytes()[:ptrsize])
+	if rodata == -1 {
+		return 0, errors.New("could not find rodata struct member")
+	}
+	// Layout of struct members is:
+	// type moduledata struct {
+	// 	...
+	// 	rodata uintptr
+	// 	gofunc uintptr
+	// 	...
+	// }
+	// So do some pointer arithmetic to get the value we need.
+	gofuncval := binary.LittleEndian.Uint64(moduleData[rodata+(1*ptrsize) : rodata+(2*ptrsize)])
+	return gofuncval, nil
+}
+
+func parseModuleData(dataSection []byte, tableAddr uint64) ([]byte, error) {
+	buf := new(bytes.Buffer)
+	err := binary.Write(buf, binary.LittleEndian, &tableAddr)
+	if err != nil {
+		return nil, err
+	}
+	off := bytes.Index(dataSection, buf.Bytes()[:4])
+	if off == -1 {
+		return nil, errors.New("could not find moduledata")
+	}
+	return dataSection[off : off+0x300], nil
+}
+
 // _STT_FUNC is a code object, see /usr/include/elf.h for a full definition.
 const _STT_FUNC = 2
 
@@ -1564,7 +1656,7 @@ func (bi *BinaryInfo) getDebugLink(exe *elf.File) (debugLink string, crc uint32)
 	}
 
 	br := gnuDebugLink.Open()
-	buf, err := ioutil.ReadAll(br)
+	buf, err := io.ReadAll(br)
 	if err != nil {
 		bi.logger.Warnf("can't read .gnu_debuglink: %v", err)
 		return
@@ -1648,6 +1740,9 @@ func (bi *BinaryInfo) setGStructOffsetElf(image *Image, exe *elf.File, wg *sync.
 
 		bi.gStructOffset = tlsg.Value + uint64(bi.Arch.PtrSize()*2) + ((tls.Vaddr - uint64(bi.Arch.PtrSize()*2)) & (tls.Align - 1))
 
+	case elf.EM_PPC64:
+		_ = getSymbol(image, bi.logger, exe, "runtime.tls_g")
+
 	default:
 		// we should never get here
 		panic("architecture not supported")
@@ -1799,20 +1894,87 @@ func loadBinaryInfoMacho(bi *BinaryInfo, image *Image, path string, entryPoint u
 	}
 
 	if entryPoint != 0 {
-		// This is a little bit hacky. We use the entryPoint variable, but it
-		// actually holds the address of the mach-o header. We can use this
-		// to calculate the offset to the non-aslr location of the mach-o header
-		// (which is 0x100000000)
-		image.StaticBase = entryPoint - 0x100000000
+		machoOff := uint64(0x100000000)
+		for _, ld := range exe.Loads {
+			if seg, _ := ld.(*macho.Segment); seg != nil {
+				if seg.Name == "__TEXT" {
+					machoOff = seg.Addr
+					break
+				}
+			}
+		}
+		logflags.DebuggerLogger().Debugf("entryPoint %#x machoOff %#x", entryPoint, machoOff)
+		image.StaticBase = entryPoint - machoOff
 	}
 
 	image.closer = exe
 	if !supportedDarwinArch[exe.Cpu] {
 		return &ErrUnsupportedArch{os: "darwin", cpuArch: exe.Cpu}
 	}
-	image.dwarf, err = exe.DWARF()
-	if err != nil {
-		return err
+	var dwerr error
+	image.dwarf, dwerr = exe.DWARF()
+	if dwerr != nil {
+		if len(bi.Images) <= 1 {
+			fmt.Fprintln(os.Stderr, "Warning: no debug info found, some functionality will be missing such as stack traces and variable evaluation.")
+		}
+		symTable, symTabAddr, err := readPcLnTableMacho(exe, path)
+		if err != nil {
+			return fmt.Errorf("could not read debug info (%v) and could not read go symbol table (%v)", dwerr, err)
+		}
+		image.symTable = symTable
+		cu := &compileUnit{}
+		cu.image = image
+		noPtrSectionData, err := exe.Section("__noptrdata").Data()
+		if err != nil {
+			return err
+		}
+		md, err := parseModuleData(noPtrSectionData, symTabAddr)
+		if err != nil {
+			return err
+		}
+		roDataAddr := exe.Section("__rodata").Addr
+		goFuncVal, err := findGoFuncVal(md, roDataAddr, bi.Arch.ptrSize)
+		if err != nil {
+			return err
+		}
+		seg := gosym.SegmentContaining(exe, goFuncVal)
+		inlFuncs := make(map[string]*Function)
+		for _, f := range image.symTable.Funcs {
+			fnEntry := f.Entry + image.StaticBase
+			if seg != nil {
+				inlCalls, err := image.symTable.GetInlineTree(&f, goFuncVal, seg.Addr, seg.ReaderAt)
+				if err != nil {
+					return err
+				}
+				for _, inlfn := range inlCalls {
+					newInlinedCall := InlinedCall{cu: cu, LowPC: fnEntry + uint64(inlfn.ParentPC)}
+					if fn, ok := inlFuncs[inlfn.Name]; ok {
+						fn.InlinedCalls = append(fn.InlinedCalls, newInlinedCall)
+						continue
+					}
+					inlFuncs[inlfn.Name] = &Function{
+						Name:  inlfn.Name,
+						Entry: 0, End: 0,
+						cu: cu,
+						InlinedCalls: []InlinedCall{
+							newInlinedCall,
+						},
+					}
+				}
+			}
+			fn := Function{Name: f.Name, Entry: fnEntry, End: f.End + image.StaticBase, cu: cu}
+			bi.Functions = append(bi.Functions, fn)
+		}
+		for i := range inlFuncs {
+			bi.Functions = append(bi.Functions, *inlFuncs[i])
+		}
+		sort.Sort(functionsDebugInfoByEntry(bi.Functions))
+		for f := range image.symTable.Files {
+			bi.Sources = append(bi.Sources, f)
+		}
+		sort.Strings(bi.Sources)
+		bi.Sources = uniq(bi.Sources)
+		return nil
 	}
 	debugInfoBytes, err := godwarf.GetDebugSectionMacho(exe, "info")
 	if err != nil {
@@ -1999,7 +2161,7 @@ func (bi *BinaryInfo) findTypeExpr(expr ast.Expr) (godwarf.Type, error) {
 	}
 	bi.expandPackagesInType(expr)
 	if snode, ok := expr.(*ast.StarExpr); ok {
-		// Pointer types only appear in the dwarf informations when
+		// Pointer types only appear in the dwarf information when
 		// a pointer to the type is used in the target program, here
 		// we create a pointer type on the fly so that the user can
 		// specify a pointer to any variable used in the target program
@@ -2137,7 +2299,7 @@ func (bi *BinaryInfo) loadDebugInfoMaps(image *Image, debugInfoBytes, debugLineB
 			if hasLineInfo && lineInfoOffset >= 0 && lineInfoOffset < int64(len(debugLineBytes)) {
 				var logfn func(string, ...interface{})
 				if logflags.DebugLineErrors() {
-					logfn = logflags.DebugLineLogger().Printf
+					logfn = logflags.DebugLineLogger().Debugf
 				}
 				cu.lineInfo = line.Parse(compdir, bytes.NewBuffer(debugLineBytes[lineInfoOffset:]), image.debugLineStr, logfn, image.StaticBase, bi.GOOS == "windows", bi.Arch.PtrSize())
 			}
@@ -2193,23 +2355,6 @@ func (bi *BinaryInfo) loadDebugInfoMaps(image *Image, debugInfoBytes, debugLineB
 	sort.Strings(bi.Sources)
 	bi.Sources = uniq(bi.Sources)
 
-	if bi.regabi {
-		// prepare patch for runtime.mallocgc's DIE
-		fn := bi.lookupOneFunc("runtime.mallocgc")
-		if fn != nil && fn.cu.image == image {
-			tree, err := image.getDwarfTree(fn.offset)
-			if err == nil {
-				children, err := regabiMallocgcWorkaround(bi)
-				if err != nil {
-					bi.logger.Errorf("could not patch runtime.mallocgc: %v", err)
-				} else {
-					tree.Children = children
-					image.runtimeMallocgcTree = tree
-				}
-			}
-		}
-	}
-
 	if cont != nil {
 		cont()
 	}
@@ -2533,7 +2678,7 @@ func (bi *BinaryInfo) loadDebugInfoMapsInlinedCalls(ctxt *loadDebugInfoMapsConte
 }
 
 func uniq(s []string) []string {
-	if len(s) <= 0 {
+	if len(s) == 0 {
 		return s
 	}
 	src, dst := 1, 1
