@@ -20,7 +20,7 @@ import (
 	"encoding/binary"
 	"fmt"
 	"io"
-	"io/ioutil"
+	"os"
 	"unicode/utf16"
 	"unsafe"
 
@@ -186,10 +186,10 @@ func (m *MemoryRange) ReadMemory(buf []byte, addr uint64) (int, error) {
 	if len(buf) == 0 {
 		return 0, nil
 	}
-	if (uint64(addr) < m.Addr) || (uint64(addr)+uint64(len(buf)) > m.Addr+uint64(len(m.Data))) {
+	if (addr < m.Addr) || (addr+uint64(len(buf)) > m.Addr+uint64(len(m.Data))) {
 		return 0, io.EOF
 	}
-	copy(buf, m.Data[uint64(addr)-m.Addr:])
+	copy(buf, m.Data[addr-m.Addr:])
 	return len(buf), nil
 }
 
@@ -318,7 +318,7 @@ const (
 
 // Open reads the minidump file at path and returns it as a Minidump structure.
 func Open(path string, logfn func(fmt string, args ...interface{})) (*Minidump, error) {
-	rawbuf, err := ioutil.ReadFile(path) //TODO(aarzilli): mmap?
+	rawbuf, err := os.ReadFile(path) //TODO(aarzilli): mmap?
 	if err != nil {
 		return nil, err
 	}
