@@ -136,6 +136,8 @@ func (t *Target) Dump(out elfwriter.WriteCloserSeeker, flags DumpFlags, state *D
 		fhdr.Machine = elf.EM_386
 	case "arm64":
 		fhdr.Machine = elf.EM_AARCH64
+	case "ppc64le":
+		fhdr.Machine = elf.EM_PPC64
 	default:
 		panic("not implemented")
 	}
