@@ -77,6 +77,8 @@ import (
 	"sync"
 	"time"
 
+	isatty "github.com/mattn/go-isatty"
+
 	"github.com/go-delve/delve/pkg/dwarf/op"
 	"github.com/go-delve/delve/pkg/elfwriter"
 	"github.com/go-delve/delve/pkg/logflags"
@@ -84,7 +86,6 @@ import (
 	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
 	"github.com/go-delve/delve/pkg/proc/linutil"
 	"github.com/go-delve/delve/pkg/proc/macutil"
-	isatty "github.com/mattn/go-isatty"
 )
 
 const (
@@ -375,11 +376,11 @@ func (p *gdbProcess) SupportsBPF() bool {
 	return false
 }
 
-func (dbp *gdbProcess) GetBufferedTracepoints() []ebpf.RawUProbeParams {
+func (p *gdbProcess) GetBufferedTracepoints() []ebpf.RawUProbeParams {
 	return nil
 }
 
-func (dbp *gdbProcess) SetUProbe(fnName string, goidOffset int64, args []ebpf.UProbeArgMap) error {
+func (p *gdbProcess) SetUProbe(fnName string, goidOffset int64, args []ebpf.UProbeArgMap) error {
 	panic("not implemented")
 }
 
@@ -588,7 +589,7 @@ func LLDBLaunch(cmd []string, wd string, flags proc.LaunchFlags, debugInfoDirs [
 // Path is path to the target's executable, path only needs to be specified
 // for some stubs that do not provide an automated way of determining it
 // (for example debugserver).
-func LLDBAttach(pid int, path string, debugInfoDirs []string) (*proc.TargetGroup, error) {
+func LLDBAttach(pid int, path string, waitFor *proc.WaitFor, debugInfoDirs []string) (*proc.TargetGroup, error) {
 	if runtime.GOOS == "windows" {
 		return nil, ErrUnsupportedOS
 	}
@@ -609,12 +610,28 @@ func LLDBAttach(pid int, path string, debugInfoDirs []string) (*proc.TargetGroup
 		if err != nil {
 			return nil, err
 		}
-		args := []string{"-R", fmt.Sprintf("127.0.0.1:%d", listener.Addr().(*net.TCPAddr).Port), "--attach=" + strconv.Itoa(pid)}
+		args := []string{"-R", fmt.Sprintf("127.0.0.1:%d", listener.Addr().(*net.TCPAddr).Port)}
+
+		if waitFor.Valid() {
+			duration := int(waitFor.Duration.Seconds())
+			if duration == 0 && waitFor.Duration != 0 {
+				// If duration is below the (second) resolution of debugserver pass 1
+				// second (0 means infinite).
+				duration = 1
+			}
+			args = append(args, "--waitfor="+waitFor.Name, fmt.Sprintf("--waitfor-interval=%d", waitFor.Interval.Microseconds()), fmt.Sprintf("--waitfor-duration=%d", duration))
+		} else {
+			args = append(args, "--attach="+strconv.Itoa(pid))
+		}
+
 		if canUnmaskSignals(debugserverExecutable) {
 			args = append(args, "--unmask-signals")
 		}
 		process = commandLogger(debugserverExecutable, args...)
 	} else {
+		if waitFor.Valid() {
+			return nil, proc.ErrWaitForNotImplemented
+		}
 		if _, err = exec.LookPath("lldb-server"); err != nil {
 			return nil, &ErrBackendUnavailable{}
 		}
@@ -646,7 +663,7 @@ func LLDBAttach(pid int, path string, debugInfoDirs []string) (*proc.TargetGroup
 // debugging PIEs.
 func (p *gdbProcess) EntryPoint() (uint64, error) {
 	var entryPoint uint64
-	if p.bi.GOOS == "darwin" && p.bi.Arch.Name == "arm64" {
+	if p.bi.GOOS == "darwin" {
 		// There is no auxv on darwin, however, we can get the location of the mach-o
 		// header from the debugserver by going through the loaded libraries, which includes
 		// the exe itself
@@ -732,7 +749,7 @@ func (p *gdbProcess) initialize(path, cmdline string, debugInfoDirs []string, st
 	})
 	_, err = addTarget(p, p.conn.pid, p.currentThread, path, stopReason, cmdline)
 	if err != nil {
-		p.Detach(true)
+		p.Detach(p.conn.pid, true)
 		return nil, err
 	}
 	return grp, nil
@@ -763,7 +780,7 @@ func (p *gdbProcess) Recorded() (bool, string) {
 
 // Pid returns the process ID.
 func (p *gdbProcess) Pid() int {
-	return int(p.conn.pid)
+	return p.conn.pid
 }
 
 // Valid returns true if we are not detached
@@ -835,7 +852,7 @@ func (p *gdbProcess) ContinueOnce(cctx *proc.ContinueOnceContext) (proc.Thread,
 		// step threads stopped at any breakpoint over their breakpoint
 		for _, thread := range p.threads {
 			if thread.CurrentBreakpoint.Breakpoint != nil {
-				if err := thread.StepInstruction(); err != nil {
+				if err := thread.stepInstruction(); err != nil {
 					return nil, proc.StopUnknown, err
 				}
 			}
@@ -1042,7 +1059,9 @@ func (p *gdbProcess) getCtrlC(cctx *proc.ContinueOnceContext) bool {
 
 // Detach will detach from the target process,
 // if 'kill' is true it will also kill the process.
-func (p *gdbProcess) Detach(kill bool) error {
+// The _pid argument is unused as follow exec
+// mode is not implemented with this backend.
+func (p *gdbProcess) Detach(_pid int, kill bool) error {
 	if kill && !p.exited {
 		err := p.conn.kill()
 		if err != nil {
@@ -1063,6 +1082,10 @@ func (p *gdbProcess) Detach(kill bool) error {
 		p.process = nil
 	}
 	p.detached = true
+	return nil
+}
+
+func (p *gdbProcess) Close() error {
 	if p.onDetach != nil {
 		p.onDetach()
 	}
@@ -1467,6 +1490,10 @@ func (p *gdbProcess) WriteMemory(addr uint64, data []byte) (written int, err err
 	return p.conn.writeMemory(addr, data)
 }
 
+func (p *gdbProcess) StepInstruction(threadID int) error {
+	return p.threads[threadID].stepInstruction()
+}
+
 func (t *gdbThread) ProcessMemory() proc.MemoryReadWriter {
 	return t.p
 }
@@ -1524,7 +1551,7 @@ func (t *gdbThread) Common() *proc.CommonThread {
 }
 
 // StepInstruction will step exactly 1 CPU instruction.
-func (t *gdbThread) StepInstruction() error {
+func (t *gdbThread) stepInstruction() error {
 	pc := t.regs.PC()
 	if bp, atbp := t.p.breakpoints.M[pc]; atbp && bp.WatchType == 0 {
 		err := t.p.conn.clearBreakpoint(pc, swBreakpoint, t.p.breakpointKind)
@@ -1961,7 +1988,7 @@ func (regs *gdbRegisters) byName(name string) uint64 {
 	return binary.LittleEndian.Uint64(reg.value)
 }
 
-func (r *gdbRegisters) FloatLoadError() error {
+func (regs *gdbRegisters) FloatLoadError() error {
 	return nil
 }
 
