@@ -13,12 +13,10 @@ import (
 	"os"
 	"strconv"
 	"strings"
-	"sync"
 	"time"
 
 	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc"
-	"github.com/sirupsen/logrus"
 )
 
 type gdbConn struct {
@@ -28,9 +26,7 @@ type gdbConn struct {
 	inbuf  []byte
 	outbuf bytes.Buffer
 
-	manualStopMutex sync.Mutex
-	running         bool
-	resumeChan      chan<- struct{}
+	running bool
 
 	direction proc.Direction // direction of execution
 
@@ -52,7 +48,7 @@ type gdbConn struct {
 
 	useXcmd bool // forces writeMemory to use the 'X' command
 
-	log *logrus.Entry
+	log logflags.Logger
 }
 
 var ErrTooManyAttempts = errors.New("too many transmit attempts")
@@ -84,7 +80,7 @@ func isProtocolErrorUnsupported(err error) bool {
 	return gdberr.code == ""
 }
 
-// GdbMalformedThreadIDError is returned when a the stub responds with a
+// GdbMalformedThreadIDError is returned when the stub responds with a
 // thread ID that does not conform with the Gdb Remote Serial Protocol
 // specification.
 type GdbMalformedThreadIDError struct {
@@ -204,7 +200,7 @@ func (conn *gdbConn) qSupported(multiprocess bool) (features map[string]bool, er
 	resp := strings.Split(string(respBuf), ";")
 	features = make(map[string]bool)
 	for _, stubfeature := range resp {
-		if len(stubfeature) <= 0 {
+		if len(stubfeature) == 0 {
 			continue
 		} else if equal := strings.Index(stubfeature, "="); equal >= 0 {
 			if stubfeature[:equal] == "PacketSize" {
@@ -259,7 +255,8 @@ func setRegFound(regFound map[string]bool, name string) {
 // readTargetXml reads target.xml file from stub using qXfer:features:read,
 // then parses it requesting any additional files.
 // The schema of target.xml is described by:
-//  https://github.com/bminor/binutils-gdb/blob/61baf725eca99af2569262d10aca03dcde2698f6/gdb/features/gdb-target.dtd
+//
+//	https://github.com/bminor/binutils-gdb/blob/61baf725eca99af2569262d10aca03dcde2698f6/gdb/features/gdb-target.dtd
 func (conn *gdbConn) readTargetXml(regFound map[string]bool) (err error) {
 	conn.regsInfo, err = conn.readAnnex("target.xml")
 	if err != nil {
@@ -327,7 +324,7 @@ func (conn *gdbConn) readRegisterInfo(regFound map[string]bool) (err error) {
 				case "container-regs":
 					contained = true
 				case "set":
-					if value == "Exception State Registers" {
+					if value == "Exception State Registers" || value == "AMX Registers" {
 						// debugserver doesn't like it if we try to write these
 						ignoreOnWrite = true
 					}
@@ -567,7 +564,7 @@ func (conn *gdbConn) writeRegister(threadID string, regnum int, data []byte) err
 // resume each thread. If a thread has sig == 0 the 'c' action will be used,
 // otherwise the 'C' action will be used and the value of sig will be passed
 // to it.
-func (conn *gdbConn) resume(threads map[int]*gdbThread, tu *threadUpdater) (stopPacket, error) {
+func (conn *gdbConn) resume(cctx *proc.ContinueOnceContext, threads map[int]*gdbThread, tu *threadUpdater) (stopPacket, error) {
 	if conn.direction == proc.Forward {
 		conn.outbuf.Reset()
 		fmt.Fprintf(&conn.outbuf, "$vCont")
@@ -584,21 +581,21 @@ func (conn *gdbConn) resume(threads map[int]*gdbThread, tu *threadUpdater) (stop
 		conn.outbuf.Reset()
 		fmt.Fprint(&conn.outbuf, "$bc")
 	}
-	conn.manualStopMutex.Lock()
+	cctx.StopMu.Lock()
 	if err := conn.send(conn.outbuf.Bytes()); err != nil {
-		conn.manualStopMutex.Unlock()
+		cctx.StopMu.Unlock()
 		return stopPacket{}, err
 	}
 	conn.running = true
-	conn.manualStopMutex.Unlock()
+	cctx.StopMu.Unlock()
 	defer func() {
-		conn.manualStopMutex.Lock()
+		cctx.StopMu.Lock()
 		conn.running = false
-		conn.manualStopMutex.Unlock()
+		cctx.StopMu.Unlock()
 	}()
-	if conn.resumeChan != nil {
-		close(conn.resumeChan)
-		conn.resumeChan = nil
+	if cctx.ResumeChan != nil {
+		close(cctx.ResumeChan)
+		cctx.ResumeChan = nil
 	}
 	return conn.waitForvContStop("resume", "-1", tu)
 }
@@ -653,8 +650,20 @@ func (conn *gdbConn) step(th *gdbThread, tu *threadUpdater, ignoreFaultSignal bo
 			if ignoreFaultSignal { // we attempting to read the TLS, a fault here should be ignored
 				return nil
 			}
+			if conn.isDebugserver {
+				// For some reason trying to deliver a signal in vCont step makes
+				// debugserver lockup (no errors, it just gets stuck), store the signal
+				// to deliver it later with the vCont;c
+				th.sig = sig
+				return nil
+			}
 		case _SIGILL, _SIGBUS, _SIGFPE:
-			// propagate these signals to inferior immediately
+			if conn.isDebugserver {
+				// See comment above
+				th.sig = sig
+				return nil
+			}
+			// otherwise propagate these signals to inferior immediately
 		case interruptSignal, breakpointSignal, stopSignal:
 			return nil
 		case childSignal: // stop on debugserver but SIGCHLD on lldb-server/linux
@@ -718,9 +727,10 @@ type stopPacket struct {
 
 // Mach exception codes used to decode metype/medata keys in stop packets (necessary to support watchpoints with debugserver).
 // See:
-//  https://opensource.apple.com/source/xnu/xnu-4570.1.46/osfmk/mach/exception_types.h.auto.html
-//  https://opensource.apple.com/source/xnu/xnu-4570.1.46/osfmk/mach/i386/exception.h.auto.html
-//  https://opensource.apple.com/source/xnu/xnu-4570.1.46/osfmk/mach/arm/exception.h.auto.html
+//
+//	https://opensource.apple.com/source/xnu/xnu-4570.1.46/osfmk/mach/exception_types.h.auto.html
+//	https://opensource.apple.com/source/xnu/xnu-4570.1.46/osfmk/mach/i386/exception.h.auto.html
+//	https://opensource.apple.com/source/xnu/xnu-4570.1.46/osfmk/mach/arm/exception.h.auto.html
 const (
 	_EXC_BREAKPOINT   = 6     // mach exception type for hardware breakpoints
 	_EXC_I386_SGL     = 1     // mach exception code for single step on x86, for some reason this is also used for watchpoints
@@ -748,24 +758,9 @@ func (conn *gdbConn) parseStopPacket(resp []byte, threadID string, tu *threadUpd
 		var metype int
 		var medata = make([]uint64, 0, 10)
 
-		buf := resp[3:]
-		for buf != nil {
-			colon := bytes.Index(buf, []byte{':'})
-			if colon < 0 {
-				break
-			}
-			key := buf[:colon]
-			buf = buf[colon+1:]
-
-			semicolon := bytes.Index(buf, []byte{';'})
-			var value []byte
-			if semicolon < 0 {
-				value = buf
-				buf = nil
-			} else {
-				value = buf[:semicolon]
-				buf = buf[semicolon+1:]
-			}
+		csp := colonSemicolonParser{buf: resp[3:]}
+		for csp.next() {
+			key, value := csp.key, csp.value
 
 			switch string(key) {
 			case "thread":
@@ -862,27 +857,15 @@ func (conn *gdbConn) queryProcessInfo(pid int) (map[string]string, error) {
 
 	pi := make(map[string]string)
 
-	for len(resp) > 0 {
-		semicolon := bytes.Index(resp, []byte{';'})
-		keyval := resp
-		if semicolon >= 0 {
-			keyval = resp[:semicolon]
-			resp = resp[semicolon+1:]
-		}
-
-		colon := bytes.Index(keyval, []byte{':'})
-		if colon < 0 {
-			continue
-		}
-
-		key := string(keyval[:colon])
-		value := string(keyval[colon+1:])
+	csp := colonSemicolonParser{buf: resp}
+	for csp.next() {
+		key, value := string(csp.key), string(csp.value)
 
 		switch key {
 		case "name":
 			name := make([]byte, len(value)/2)
 			for i := 0; i < len(value); i += 2 {
-				n, _ := strconv.ParseUint(string(value[i:i+2]), 16, 8)
+				n, _ := strconv.ParseUint(value[i:i+2], 16, 8)
 				name[i/2] = byte(n)
 			}
 			pi[key] = string(name)
@@ -1098,6 +1081,36 @@ func (conn *gdbConn) threadStopInfo(threadID string) (sp stopPacket, err error)
 	if err != nil {
 		return stopPacket{}, err
 	}
+	if sp.threadID != threadID {
+		// When we send a ^C (manual stop request) and the process is close to
+		// stopping anyway, sometimes, debugserver will send back two stop
+		// packets. We need to ignore this spurious stop packet. Because the first
+		// thing we do after the stop is updateThreadList, which calls this
+		// function, this is relatively painless. We simply need to check that the
+		// stop packet we receive is for the thread we requested, if it isn't we
+		// can assume it is the spurious extra stop packet and simply ignore it.
+		// An example of a problematic interaction is in the commit message for
+		// this change.
+		// See https://github.com/go-delve/delve/issues/3013.
+
+		conn.conn.SetReadDeadline(time.Now().Add(10 * time.Millisecond))
+		resp, err = conn.recv(conn.outbuf.Bytes(), "thread stop info", false)
+		conn.conn.SetReadDeadline(time.Time{})
+		if err != nil {
+			if neterr, isneterr := err.(net.Error); isneterr && neterr.Timeout() {
+				return stopPacket{}, fmt.Errorf("qThreadStopInfo mismatch, requested %s got %s", sp.threadID, threadID)
+			}
+			return stopPacket{}, err
+		}
+		_, sp, err = conn.parseStopPacket(resp, "", nil)
+		if err != nil {
+			return stopPacket{}, err
+		}
+		if sp.threadID != threadID {
+			return stopPacket{}, fmt.Errorf("qThreadStopInfo mismatch, requested %s got %s", sp.threadID, threadID)
+		}
+	}
+
 	return sp, nil
 }
 
@@ -1195,24 +1208,9 @@ func (conn *gdbConn) memoryRegionInfo(addr uint64) (*memoryRegionInfo, error) {
 
 	mri := &memoryRegionInfo{}
 
-	buf := resp
-	for len(buf) > 0 {
-		colon := bytes.Index(buf, []byte{':'})
-		if colon < 0 {
-			break
-		}
-		key := buf[:colon]
-		buf = buf[colon+1:]
-
-		semicolon := bytes.Index(buf, []byte{';'})
-		var value []byte
-		if semicolon < 0 {
-			value = buf
-			buf = nil
-		} else {
-			value = buf[:semicolon]
-			buf = buf[semicolon+1:]
-		}
+	csp := colonSemicolonParser{buf: resp}
+	for csp.next() {
+		key, value := csp.key, csp.value
 
 		switch string(key) {
 		case "start":
@@ -1249,7 +1247,8 @@ func (conn *gdbConn) memoryRegionInfo(addr uint64) (*memoryRegionInfo, error) {
 
 // exec executes a message to the stub and reads a response.
 // The details of the wire protocol are described here:
-//  https://sourceware.org/gdb/onlinedocs/gdb/Overview.html#Overview
+//
+//	https://sourceware.org/gdb/onlinedocs/gdb/Overview.html#Overview
 func (conn *gdbConn) exec(cmd []byte, context string) ([]byte, error) {
 	if err := conn.send(cmd); err != nil {
 		return nil, err
@@ -1257,7 +1256,7 @@ func (conn *gdbConn) exec(cmd []byte, context string) ([]byte, error) {
 	return conn.recv(cmd, context, false)
 }
 
-var hexdigit = []byte{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'}
+const hexdigit = "0123456789abcdef"
 
 func (conn *gdbConn) send(cmd []byte) error {
 	if len(cmd) == 0 || cmd[0] != '$' {
@@ -1315,9 +1314,11 @@ func (conn *gdbConn) recv(cmd []byte, context string, binary bool) (resp []byte,
 		if logflags.GdbWire() {
 			out := resp
 			partial := false
-			if idx := bytes.Index(out, []byte{'\n'}); idx >= 0 && !binary {
-				out = resp[:idx]
-				partial = true
+			if !binary {
+				if idx := bytes.Index(out, []byte{'\n'}); idx >= 0 {
+					out = resp[:idx]
+					partial = true
+				}
 			}
 			if len(out) > gdbWireMaxLen {
 				out = out[:gdbWireMaxLen]
@@ -1480,7 +1481,7 @@ func binarywiredecode(in, buf []byte) (newbuf, msg []byte) {
 	return buf, buf[start:]
 }
 
-// Checksumok checks that checksum is a valid checksum for packet.
+// checksumok checks that checksum is a valid checksum for packet.
 func checksumok(packet, checksumBuf []byte) bool {
 	if packet[0] != '$' {
 		return false
@@ -1506,3 +1507,33 @@ func checksum(packet []byte) (sum uint8) {
 	}
 	return sum
 }
+
+// colonSemicolonParser parses a string in the form:
+//
+//	key1:value1;key2:value2;...;keyN:valueN
+type colonSemicolonParser struct {
+	buf        []byte
+	key, value []byte
+}
+
+func (csp *colonSemicolonParser) next() bool {
+	if len(csp.buf) == 0 {
+		return false
+	}
+	colon := bytes.IndexByte(csp.buf, ':')
+	if colon < 0 {
+		return false
+	}
+	csp.key = csp.buf[:colon]
+	csp.buf = csp.buf[colon+1:]
+
+	semicolon := bytes.IndexByte(csp.buf, ';')
+	if semicolon < 0 {
+		csp.value = csp.buf
+		csp.buf = nil
+	} else {
+		csp.value = csp.buf[:semicolon]
+		csp.buf = csp.buf[semicolon+1:]
+	}
+	return true
+}
