@@ -98,17 +98,17 @@ type compositeMemory struct {
 
 // CreateCompositeMemory created a new composite memory type using the provided MemoryReadWriter as the
 // underlying memory buffer.
-func CreateCompositeMemory(mem MemoryReadWriter, arch *Arch, regs op.DwarfRegisters, pieces []op.Piece) (*compositeMemory, error) {
+func CreateCompositeMemory(mem MemoryReadWriter, arch *Arch, regs op.DwarfRegisters, pieces []op.Piece, size int64) (*compositeMemory, error) {
 	// This is basically a small wrapper to avoid having to change all callers
 	// of newCompositeMemory since it existed first.
-	cm, err := newCompositeMemory(mem, arch, regs, pieces)
+	cm, err := newCompositeMemory(mem, arch, regs, pieces, size)
 	if cm != nil {
 		cm.base = fakeAddressUnresolv
 	}
 	return cm, err
 }
 
-func newCompositeMemory(mem MemoryReadWriter, arch *Arch, regs op.DwarfRegisters, pieces []op.Piece) (*compositeMemory, error) {
+func newCompositeMemory(mem MemoryReadWriter, arch *Arch, regs op.DwarfRegisters, pieces []op.Piece, size int64) (*compositeMemory, error) {
 	cmem := &compositeMemory{realmem: mem, arch: arch, regs: regs, pieces: pieces, data: []byte{}}
 	for i := range pieces {
 		piece := &pieces[i]
@@ -127,7 +127,7 @@ func newCompositeMemory(mem MemoryReadWriter, arch *Arch, regs op.DwarfRegisters
 			cmem.data = append(cmem.data, reg[:piece.Size]...)
 		case op.AddrPiece:
 			buf := make([]byte, piece.Size)
-			mem.ReadMemory(buf, uint64(piece.Val))
+			mem.ReadMemory(buf, piece.Val)
 			cmem.data = append(cmem.data, buf...)
 		case op.ImmPiece:
 			buf := piece.Bytes
@@ -147,6 +147,11 @@ func newCompositeMemory(mem MemoryReadWriter, arch *Arch, regs op.DwarfRegisters
 			panic("unsupported piece kind")
 		}
 	}
+	paddingBytes := int(size) - len(cmem.data)
+	if paddingBytes > 0 && paddingBytes < arch.ptrSize {
+		padding := make([]byte, paddingBytes)
+		cmem.data = append(cmem.data, padding...)
+	}
 	return cmem, nil
 }
 
@@ -186,7 +191,7 @@ func (mem *compositeMemory) WriteMemory(addr uint64, data []byte) (int, error) {
 					return donesz, err
 				}
 			case op.AddrPiece:
-				n, err := mem.realmem.WriteMemory(uint64(piece.Val), pieceMem)
+				n, err := mem.realmem.WriteMemory(piece.Val, pieceMem)
 				if err != nil {
 					return donesz + n, err
 				}
