@@ -6,7 +6,6 @@ import (
 	"debug/elf"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"os/signal"
@@ -19,6 +18,7 @@ import (
 
 	sys "golang.org/x/sys/unix"
 
+	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc"
 	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
 	"github.com/go-delve/delve/pkg/proc/linutil"
@@ -62,7 +62,7 @@ func (os *osProcessDetails) Close() {
 // to be supplied to that process. `wd` is working directory of the program.
 // If the DWARF information cannot be found in the binary, Delve will look
 // for external debug files in the directories passed in.
-func Launch(cmd []string, wd string, flags proc.LaunchFlags, debugInfoDirs []string, tty string, redirects [3]string) (*proc.TargetGroup, error) {
+func Launch(cmd []string, wd string, flags proc.LaunchFlags, debugInfoDirs []string, tty string, stdinPath string, stdoutOR proc.OutputRedirect, stderrOR proc.OutputRedirect) (*proc.TargetGroup, error) {
 	var (
 		process *exec.Cmd
 		err     error
@@ -70,7 +70,7 @@ func Launch(cmd []string, wd string, flags proc.LaunchFlags, debugInfoDirs []str
 
 	foreground := flags&proc.LaunchForeground != 0
 
-	stdin, stdout, stderr, closefn, err := openRedirects(redirects, foreground)
+	stdin, stdout, stderr, closefn, err := openRedirects(stdinPath, stdoutOR, stderrOR, foreground)
 	if err != nil {
 		return nil, err
 	}
@@ -84,7 +84,7 @@ func Launch(cmd []string, wd string, flags proc.LaunchFlags, debugInfoDirs []str
 	dbp := newProcess(0)
 	defer func() {
 		if err != nil && dbp.pid != 0 {
-			_ = dbp.Detach(true)
+			_ = detachWithoutGroup(dbp, true)
 		}
 	}()
 	dbp.execPtraceFunc(func() {
@@ -141,7 +141,15 @@ func Launch(cmd []string, wd string, flags proc.LaunchFlags, debugInfoDirs []str
 // Attach to an existing process with the given PID. Once attached, if
 // the DWARF information cannot be found in the binary, Delve will look
 // for external debug files in the directories passed in.
-func Attach(pid int, debugInfoDirs []string) (*proc.TargetGroup, error) {
+func Attach(pid int, waitFor *proc.WaitFor, debugInfoDirs []string) (*proc.TargetGroup, error) {
+	if waitFor.Valid() {
+		var err error
+		pid, err = WaitFor(waitFor)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	dbp := newProcess(pid)
 
 	var err error
@@ -156,7 +164,7 @@ func Attach(pid int, debugInfoDirs []string) (*proc.TargetGroup, error) {
 
 	tgt, err := dbp.initialize(findExecutable("", dbp.pid), debugInfoDirs)
 	if err != nil {
-		_ = dbp.Detach(false)
+		_ = detachWithoutGroup(dbp, false)
 		return nil, err
 	}
 
@@ -169,15 +177,62 @@ func Attach(pid int, debugInfoDirs []string) (*proc.TargetGroup, error) {
 	return tgt, nil
 }
 
+func isProcDir(name string) bool {
+	for _, ch := range name {
+		if ch < '0' || ch > '9' {
+			return false
+		}
+	}
+	return true
+}
+
+func waitForSearchProcess(pfx string, seen map[int]struct{}) (int, error) {
+	log := logflags.DebuggerLogger()
+	des, err := os.ReadDir("/proc")
+	if err != nil {
+		log.Errorf("error reading proc: %v", err)
+		return 0, nil
+	}
+	for _, de := range des {
+		if !de.IsDir() {
+			continue
+		}
+		name := de.Name()
+		if !isProcDir(name) {
+			continue
+		}
+		pid, _ := strconv.Atoi(name)
+		if _, isseen := seen[pid]; isseen {
+			continue
+		}
+		seen[pid] = struct{}{}
+		buf, err := os.ReadFile(filepath.Join("/proc", name, "cmdline"))
+		if err != nil {
+			// probably we just don't have permissions
+			continue
+		}
+		for i := range buf {
+			if buf[i] == 0 {
+				buf[i] = ' '
+			}
+		}
+		log.Debugf("waitfor: new process %q", string(buf))
+		if strings.HasPrefix(string(buf), pfx) {
+			return pid, nil
+		}
+	}
+	return 0, nil
+}
+
 func initialize(dbp *nativeProcess) (string, error) {
-	comm, err := ioutil.ReadFile(fmt.Sprintf("/proc/%d/comm", dbp.pid))
+	comm, err := os.ReadFile(fmt.Sprintf("/proc/%d/comm", dbp.pid))
 	if err == nil {
 		// removes newline character
 		comm = bytes.TrimSuffix(comm, []byte("\n"))
 	}
 
 	if comm == nil || len(comm) <= 0 {
-		stat, err := ioutil.ReadFile(fmt.Sprintf("/proc/%d/stat", dbp.pid))
+		stat, err := os.ReadFile(fmt.Sprintf("/proc/%d/stat", dbp.pid))
 		if err != nil {
 			return "", fmt.Errorf("could not read proc stat: %v", err)
 		}
@@ -205,8 +260,8 @@ func (dbp *nativeProcess) GetBufferedTracepoints() []ebpf.RawUProbeParams {
 }
 
 // kill kills the target process.
-func (dbp *nativeProcess) kill() error {
-	if dbp.exited {
+func (procgrp *processGroup) kill(dbp *nativeProcess) error {
+	if ok, _ := dbp.Valid(); !ok {
 		return nil
 	}
 	if !dbp.threads[dbp.pid].Stopped() {
@@ -434,7 +489,7 @@ func trapWaitInternal(procgrp *processGroup, pid int, options trapWaitOptions) (
 				dbp.threads[int(wpid)].os.running = false
 				return nil, nil
 			}
-			if err = th.Continue(); err != nil {
+			if err = th.resume(); err != nil {
 				if err == sys.ESRCH {
 					// thread died while we were adding it
 					delete(dbp.threads, th.ID)
@@ -442,7 +497,7 @@ func trapWaitInternal(procgrp *processGroup, pid int, options trapWaitOptions) (
 				}
 				return nil, fmt.Errorf("could not continue new thread %d %s", cloned, err)
 			}
-			if err = dbp.threads[int(wpid)].Continue(); err != nil {
+			if err = dbp.threads[int(wpid)].resume(); err != nil {
 				if err != sys.ESRCH {
 					return nil, fmt.Errorf("could not continue existing thread %d %s", wpid, err)
 				}
@@ -463,7 +518,6 @@ func trapWaitInternal(procgrp *processGroup, pid int, options trapWaitOptions) (
 			cmdline, _ := dbp.initializeBasic()
 			tgt, err := procgrp.add(dbp, dbp.pid, dbp.memthread, findExecutable("", dbp.pid), proc.StopLaunched, cmdline)
 			if err != nil {
-				_ = dbp.Detach(false)
 				return nil, err
 			}
 			if halt {
@@ -472,7 +526,7 @@ func trapWaitInternal(procgrp *processGroup, pid int, options trapWaitOptions) (
 			if tgt != nil {
 				// If tgt is nil we decided we are not interested in debugging this
 				// process, and we have already detached from it.
-				err = dbp.threads[dbp.pid].Continue()
+				err = dbp.threads[dbp.pid].resume()
 				if err != nil {
 					return nil, err
 				}
@@ -509,9 +563,23 @@ func trapWaitInternal(procgrp *processGroup, pid int, options trapWaitOptions) (
 			}
 			// do the same thing we do if a thread quit
 			if wpid == dbp.pid {
+				exitStatus := 0
+				if procgrp.numValid() == 1 {
+					// try to recover the real exit status using waitpid
+					for {
+						wpid2, status2, err := dbp.wait(-1, sys.WNOHANG)
+						if wpid2 <= 0 || err != nil {
+							break
+						}
+						if status2.Exited() {
+							exitStatus = status2.ExitStatus()
+						}
+					}
+
+				}
 				dbp.postExit()
 				if procgrp.numValid() == 0 {
-					return nil, proc.ErrProcessExited{Pid: wpid, Status: status.ExitStatus()}
+					return nil, proc.ErrProcessExited{Pid: wpid, Status: exitStatus}
 				}
 				continue
 			}
@@ -592,20 +660,28 @@ func exitGuard(dbp *nativeProcess, procgrp *processGroup, err error) error {
 	return err
 }
 
-func (dbp *nativeProcess) resume() error {
+func (procgrp *processGroup) resume() error {
 	// all threads stopped over a breakpoint are made to step over it
-	for _, thread := range dbp.threads {
-		if thread.CurrentBreakpoint.Breakpoint != nil {
-			if err := thread.StepInstruction(); err != nil {
-				return err
+	for _, dbp := range procgrp.procs {
+		if valid, _ := dbp.Valid(); valid {
+			for _, thread := range dbp.threads {
+				if thread.CurrentBreakpoint.Breakpoint != nil {
+					if err := procgrp.stepInstruction(thread); err != nil {
+						return err
+					}
+					thread.CurrentBreakpoint.Clear()
+				}
 			}
-			thread.CurrentBreakpoint.Clear()
 		}
 	}
 	// everything is resumed
-	for _, thread := range dbp.threads {
-		if err := thread.resume(); err != nil && err != sys.ESRCH {
-			return err
+	for _, dbp := range procgrp.procs {
+		if valid, _ := dbp.Valid(); valid {
+			for _, thread := range dbp.threads {
+				if err := thread.resume(); err != nil && err != sys.ESRCH {
+					return err
+				}
+			}
 		}
 	}
 	return nil
@@ -618,7 +694,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	}
 
 	for _, dbp := range procgrp.procs {
-		if dbp.exited {
+		if ok, _ := dbp.Valid(); !ok {
 			continue
 		}
 		for _, th := range dbp.threads {
@@ -631,8 +707,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	for {
 		th, err := trapWaitInternal(procgrp, -1, trapWaitNohang)
 		if err != nil {
-			p := procgrp.procForThread(th.ID)
-			return nil, exitGuard(p, procgrp, err)
+			return nil, exitGuard(procgrp.procs[0], procgrp, err)
 		}
 		if th == nil {
 			break
@@ -641,7 +716,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 
 	// stop all threads that are still running
 	for _, dbp := range procgrp.procs {
-		if dbp.exited {
+		if ok, _ := dbp.Valid(); !ok {
 			continue
 		}
 		for _, th := range dbp.threads {
@@ -662,7 +737,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	for {
 		allstopped := true
 		for _, dbp := range procgrp.procs {
-			if dbp.exited {
+			if ok, _ := dbp.Valid(); !ok {
 				continue
 			}
 			for _, th := range dbp.threads {
@@ -684,7 +759,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	switchTrapthread := false
 
 	for _, dbp := range procgrp.procs {
-		if dbp.exited {
+		if ok, _ := dbp.Valid(); !ok {
 			continue
 		}
 		err := stop1(cctx, dbp, trapthread, &switchTrapthread)
@@ -697,7 +772,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 		trapthreadID := trapthread.ID
 		trapthread = nil
 		for _, dbp := range procgrp.procs {
-			if dbp.exited {
+			if ok, _ := dbp.Valid(); !ok {
 				continue
 			}
 			for _, th := range dbp.threads {
@@ -782,6 +857,10 @@ func stop1(cctx *proc.ContinueOnceContext, dbp *nativeProcess, trapthread *nativ
 	return err1
 }
 
+func (procgrp *processGroup) detachChild(dbp *nativeProcess) error {
+	return procgrp.Detach(dbp.pid, false)
+}
+
 func (dbp *nativeProcess) detach(kill bool) error {
 	for threadID := range dbp.threads {
 		err := ptraceDetach(threadID, 0)
@@ -806,7 +885,7 @@ func (dbp *nativeProcess) detach(kill bool) error {
 // EntryPoint will return the process entry point address, useful for
 // debugging PIEs.
 func (dbp *nativeProcess) EntryPoint() (uint64, error) {
-	auxvbuf, err := ioutil.ReadFile(fmt.Sprintf("/proc/%d/auxv", dbp.pid))
+	auxvbuf, err := os.ReadFile(fmt.Sprintf("/proc/%d/auxv", dbp.pid))
 	if err != nil {
 		return 0, fmt.Errorf("could not read auxiliary vector: %v", err)
 	}
@@ -833,7 +912,7 @@ func (dbp *nativeProcess) SetUProbe(fnName string, goidOffset int64, args []ebpf
 
 	fns := dbp.bi.LookupFunc()[fnName]
 	if len(fns) != 1 {
-		return fmt.Errorf("could not find function: %s", fnName)
+		return &proc.ErrFunctionNotFound{FuncName: fnName}
 	}
 	fn := fns[0]
 
@@ -923,7 +1002,7 @@ func killProcess(pid int) error {
 }
 
 func getCmdLine(pid int) string {
-	buf, _ := ioutil.ReadFile(fmt.Sprintf("/proc/%d/cmdline", pid))
+	buf, _ := os.ReadFile(fmt.Sprintf("/proc/%d/cmdline", pid))
 	args := strings.SplitN(string(buf), "\x00", -1)
 	for i := range args {
 		if strings.Contains(args[i], " ") {
