@@ -50,7 +50,7 @@ func (t *nativeThread) resumeWithSig(sig int) (err error) {
 	return
 }
 
-func (t *nativeThread) singleStep() (err error) {
+func (procgrp *processGroup) singleStep(t *nativeThread) (err error) {
 	sig := 0
 	for {
 		t.dbp.execPtraceFunc(func() { err = ptraceSingleStep(t.ID, sig) })
@@ -88,8 +88,8 @@ func (t *nativeThread) singleStep() (err error) {
 }
 
 func (t *nativeThread) WriteMemory(addr uint64, data []byte) (written int, err error) {
-	if t.dbp.exited {
-		return 0, proc.ErrProcessExited{Pid: t.dbp.pid}
+	if ok, err := t.dbp.Valid(); !ok {
+		return 0, err
 	}
 	if len(data) == 0 {
 		return
@@ -106,8 +106,8 @@ func (t *nativeThread) WriteMemory(addr uint64, data []byte) (written int, err e
 }
 
 func (t *nativeThread) ReadMemory(data []byte, addr uint64) (n int, err error) {
-	if t.dbp.exited {
-		return 0, proc.ErrProcessExited{Pid: t.dbp.pid}
+	if ok, err := t.dbp.Valid(); !ok {
+		return 0, err
 	}
 	if len(data) == 0 {
 		return
@@ -123,23 +123,3 @@ func (t *nativeThread) ReadMemory(data []byte, addr uint64) (n int, err error) {
 func (t *nativeThread) SoftExc() bool {
 	return t.os.setbp
 }
-
-// Continue the execution of this thread.
-//
-// If we are currently at a breakpoint, we'll clear it
-// first and then resume execution. Thread will continue until
-// it hits a breakpoint or is signaled.
-func (t *nativeThread) Continue() error {
-	pc, err := t.PC()
-	if err != nil {
-		return err
-	}
-	// Check whether we are stopped at a breakpoint, and
-	// if so, single step over it before continuing.
-	if _, ok := t.dbp.FindBreakpoint(pc, false); ok {
-		if err := t.StepInstruction(); err != nil {
-			return err
-		}
-	}
-	return t.resume()
-}
