@@ -19,7 +19,7 @@ import (
 )
 
 func TestScopeWithEscapedVariable(t *testing.T) {
-	if ver, _ := goversion.Parse(runtime.Version()); ver.Major >= 0 && !ver.AfterOrEqual(goversion.GoVersion{Major: 1, Minor: 9, Rev: -1, Beta: 3}) {
+	if ver, _ := goversion.Parse(runtime.Version()); ver.Major >= 0 && !ver.AfterOrEqual(goversion.GoVersion{Major: 1, Minor: 9}) {
 		return
 	}
 
@@ -187,7 +187,7 @@ func (check *scopeCheck) Parse(descr string, t *testing.T) {
 		}
 		varcheck.name = strings.TrimSpace(decl[:space])
 		varcheck.typ = strings.TrimSpace(decl[space+1:])
-		if strings.Index(varcheck.typ, " ") >= 0 {
+		if strings.Contains(varcheck.typ, " ") {
 			t.Fatalf("could not parse scope comment %q (%q)", descr, decl)
 		}
 
@@ -237,7 +237,7 @@ func (check *scopeCheck) Parse(descr string, t *testing.T) {
 	}
 }
 
-func (scopeCheck *scopeCheck) checkLocalsAndArgs(p *proc.Target, t *testing.T) (*proc.EvalScope, bool) {
+func (check *scopeCheck) checkLocalsAndArgs(p *proc.Target, t *testing.T) (*proc.EvalScope, bool) {
 	scope, err := proc.GoroutineScope(p, p.CurrentThread())
 	assertNoError(err, t, "GoroutineScope()")
 
@@ -249,16 +249,16 @@ func (scopeCheck *scopeCheck) checkLocalsAndArgs(p *proc.Target, t *testing.T) (
 	assertNoError(err, t, "LocalVariables()")
 
 	for _, arg := range args {
-		scopeCheck.checkVar(arg, t)
+		check.checkVar(arg, t)
 	}
 
 	for _, local := range locals {
-		scopeCheck.checkVar(local, t)
+		check.checkVar(local, t)
 	}
 
-	for i := range scopeCheck.varChecks {
-		if !scopeCheck.varChecks[i].ok {
-			t.Errorf("%d: variable %s not found", scopeCheck.line, scopeCheck.varChecks[i].name)
+	for i := range check.varChecks {
+		if !check.varChecks[i].ok {
+			t.Errorf("%d: variable %s not found", check.line, check.varChecks[i].name)
 			ok = false
 		}
 	}
