@@ -9,6 +9,7 @@ import (
 	"go/constant"
 	"go/token"
 	"math"
+	"math/bits"
 	"reflect"
 	"sort"
 	"strconv"
@@ -50,7 +51,7 @@ const (
 	FloatIsNormal floatSpecial = iota
 	// FloatIsNaN means the float is a special NaN value.
 	FloatIsNaN
-	// FloatIsPosInf means the float is a special positive inifitiy value.
+	// FloatIsPosInf means the float is a special positive infinity value.
 	FloatIsPosInf
 	// FloatIsNegInf means the float is a special negative infinity value.
 	FloatIsNegInf
@@ -85,6 +86,9 @@ const (
 	VariableCPtr
 	// VariableCPURegister means this variable is a CPU register.
 	VariableCPURegister
+	// variableTrustLen means that when this variable is loaded its length
+	// should be trusted and used instead of MaxArrayValues
+	variableTrustLen
 )
 
 // Variable represents a variable. It contains the address, name,
@@ -497,7 +501,7 @@ func (g *G) Defer() *Defer {
 // UserCurrent returns the location the users code is at,
 // or was at before entering a runtime function.
 func (g *G) UserCurrent() Location {
-	it, err := g.stackIterator(0)
+	it, err := goroutineStackIterator(nil, g, 0)
 	if err != nil {
 		return g.CurrentLoc
 	}
@@ -958,7 +962,7 @@ func (v *Variable) parseG() (*G, error) {
 		SP:         uint64(sp),
 		BP:         uint64(bp),
 		LR:         uint64(lr),
-		Status:     uint64(status),
+		Status:     status,
 		WaitSince:  waitSince,
 		WaitReason: waitReason,
 		CurrentLoc: Location{PC: uint64(pc), File: f, Line: l, Fn: fn},
@@ -1191,7 +1195,7 @@ func extractVarInfoFromEntry(tgt *Target, bi *BinaryInfo, image *Image, regs op.
 	t, err = resolveParametricType(bi, mem, t, dictAddr)
 	if err != nil {
 		// Log the error, keep going with t, which will be the shape type
-		logflags.DebuggerLogger().Errorf("could not resolve parametric type of %s", n)
+		logflags.DebuggerLogger().Errorf("could not resolve parametric type of %s: %v", n, err)
 	}
 
 	addr, pieces, descr, err := bi.Location(entry, dwarf.AttrLocation, regs.PC(), regs, mem)
@@ -1305,10 +1309,21 @@ func (v *Variable) loadValueInternal(recurseLevel int, cfg LoadConfig) {
 			// Don't increase the recursion level when dereferencing pointers
 			// unless this is a pointer to interface (which could cause an infinite loop)
 			nextLvl := recurseLevel
+			checkLvl := false
 			if v.Children[0].Kind == reflect.Interface {
 				nextLvl++
+			} else if ptyp, isptr := v.RealType.(*godwarf.PtrType); isptr {
+				_, elemTypIsPtr := resolveTypedef(ptyp.Type).(*godwarf.PtrType)
+				if elemTypIsPtr {
+					nextLvl++
+					checkLvl = true
+				}
+			}
+			if checkLvl && recurseLevel > cfg.MaxVariableRecurse {
+				v.Children[0].OnlyAddr = true
+			} else {
+				v.Children[0].loadValueInternal(nextLvl, cfg)
 			}
-			v.Children[0].loadValueInternal(nextLvl, cfg)
 		} else {
 			v.Children[0].OnlyAddr = true
 		}
@@ -1422,7 +1437,7 @@ func (v *Variable) loadValueInternal(recurseLevel int, cfg LoadConfig) {
 	case reflect.Func:
 		v.readFunctionPtr()
 	default:
-		v.Unreadable = fmt.Errorf("unknown or unsupported kind: \"%s\"", v.Kind.String())
+		v.Unreadable = fmt.Errorf("unknown or unsupported kind: %q", v.Kind.String())
 	}
 }
 
@@ -1442,7 +1457,7 @@ func convertToEface(srcv, dstv *Variable) error {
 			return srcv.Unreadable
 		}
 		_type = _type.maybeDereference()
-		dstv.writeEmptyInterface(uint64(_type.Addr), data)
+		dstv.writeEmptyInterface(_type.Addr, data)
 		return nil
 	}
 	typeAddr, typeKind, runtimeTypeFound, err := dwarfToRuntimeType(srcv.bi, srcv.mem, srcv.RealType)
@@ -1671,9 +1686,12 @@ func (v *Variable) loadArrayValues(recurseLevel int, cfg LoadConfig) {
 
 	count := v.Len
 	// Cap number of elements
-	if count > int64(cfg.MaxArrayValues) {
+	if (v.Flags&variableTrustLen == 0) && (count > int64(cfg.MaxArrayValues)) {
 		count = int64(cfg.MaxArrayValues)
 	}
+	if v.Base+uint64(v.stride*count) < v.Base {
+		v.Unreadable = fmt.Errorf("bad array base address %#x", v.Base)
+	}
 
 	if v.stride < maxArrayStridePrefetch {
 		v.mem = cacheMemory(v.mem, v.Base, int(v.stride*count))
@@ -1713,7 +1731,7 @@ func (v *Variable) readComplex(size int64) {
 		return
 	}
 
-	ftyp := fakeBasicType("float", int(fs*8))
+	ftyp := godwarf.FakeBasicType("float", int(fs*8))
 
 	realvar := v.newVariable("real", v.Addr, ftyp, v.mem)
 	imagvar := v.newVariable("imaginary", v.Addr+uint64(fs), ftyp, v.mem)
@@ -1723,13 +1741,13 @@ func (v *Variable) readComplex(size int64) {
 }
 
 func (v *Variable) writeComplex(real, imag float64, size int64) error {
-	err := v.writeFloatRaw(real, int64(size/2))
+	err := v.writeFloatRaw(real, size/2)
 	if err != nil {
 		return err
 	}
 	imagaddr := *v
 	imagaddr.Addr += uint64(size / 2)
-	return imagaddr.writeFloatRaw(imag, int64(size/2))
+	return imagaddr.writeFloatRaw(imag, size/2)
 }
 
 func readIntRaw(mem MemoryReadWriter, addr uint64, size int64) (int64, error) {
@@ -1766,7 +1784,7 @@ func (v *Variable) writeUint(value uint64, size int64) error {
 	case 4:
 		binary.LittleEndian.PutUint32(val, uint32(value))
 	case 8:
-		binary.LittleEndian.PutUint64(val, uint64(value))
+		binary.LittleEndian.PutUint64(val, value)
 	}
 
 	_, err := v.mem.WriteMemory(v.Addr, val)
@@ -1790,7 +1808,7 @@ func readUintRaw(mem MemoryReadWriter, addr uint64, size int64) (uint64, error)
 	case 4:
 		n = uint64(binary.LittleEndian.Uint32(val))
 	case 8:
-		n = uint64(binary.LittleEndian.Uint64(val))
+		n = binary.LittleEndian.Uint64(val)
 	}
 
 	return n, nil
@@ -1826,7 +1844,7 @@ func (v *Variable) writeFloatRaw(f float64, size int64) error {
 		n := float32(f)
 		binary.Write(buf, binary.LittleEndian, n)
 	case 8:
-		n := float64(f)
+		n := f
 		binary.Write(buf, binary.LittleEndian, n)
 	}
 
@@ -1863,7 +1881,7 @@ func (v *Variable) writeSlice(len, cap int64, base uint64) error {
 		switch f.Name {
 		case sliceArrayFieldName:
 			arrv, _ := v.toField(f)
-			if err := arrv.writeUint(uint64(base), arrv.RealType.Size()); err != nil {
+			if err := arrv.writeUint(base, arrv.RealType.Size()); err != nil {
 				return err
 			}
 		case sliceLenFieldName:
@@ -1882,8 +1900,8 @@ func (v *Variable) writeSlice(len, cap int64, base uint64) error {
 }
 
 func (v *Variable) writeString(len, base uint64) error {
-	writePointer(v.bi, v.mem, uint64(v.Addr), base)
-	writePointer(v.bi, v.mem, uint64(v.Addr)+uint64(v.bi.Arch.PtrSize()), len)
+	writePointer(v.bi, v.mem, v.Addr, base)
+	writePointer(v.bi, v.mem, v.Addr+uint64(v.bi.Arch.PtrSize()), len)
 	return nil
 }
 
@@ -1916,7 +1934,7 @@ func (v *Variable) readFunctionPtr() {
 	}
 
 	v.Base = val
-	fn := v.bi.PCToFunc(uint64(v.Base))
+	fn := v.bi.PCToFunc(v.Base)
 	if fn == nil {
 		v.Unreadable = fmt.Errorf("could not find function for %#v", v.Base)
 		return
@@ -2391,7 +2409,7 @@ func (v *Variable) registerVariableTypeConv(newtyp string) (*Variable, error) {
 			child.Kind = reflect.Uint32
 			n = 4
 		case "uint64":
-			child = newConstant(constant.MakeUint64(uint64(binary.LittleEndian.Uint64(v.reg.Bytes[i:]))), v.mem)
+			child = newConstant(constant.MakeUint64(binary.LittleEndian.Uint64(v.reg.Bytes[i:])), v.mem)
 			child.Kind = reflect.Uint64
 			n = 8
 		case "float32":
@@ -2414,7 +2432,7 @@ func (v *Variable) registerVariableTypeConv(newtyp string) (*Variable, error) {
 						break
 					}
 				}
-				if n == 0 || popcnt(uint64(n)) != 1 {
+				if n == 0 || bits.OnesCount64(uint64(n)) != 1 {
 					return nil, fmt.Errorf("unknown CPU register type conversion to %q", newtyp)
 				}
 				n = n / 8
@@ -2433,23 +2451,6 @@ func (v *Variable) registerVariableTypeConv(newtyp string) (*Variable, error) {
 	return v, nil
 }
 
-// popcnt is the number of bits set to 1 in x.
-// It's the same as math/bits.OnesCount64, copied here so that we can build
-// on versions of go that don't have math/bits.
-func popcnt(x uint64) int {
-	const m0 = 0x5555555555555555 // 01010101 ...
-	const m1 = 0x3333333333333333 // 00110011 ...
-	const m2 = 0x0f0f0f0f0f0f0f0f // 00001111 ...
-	const m = 1<<64 - 1
-	x = x>>1&(m0&m) + x&(m0&m)
-	x = x>>2&(m1&m) + x&(m1&m)
-	x = (x>>4 + x) & (m2 & m)
-	x += x >> 8
-	x += x >> 16
-	x += x >> 32
-	return int(x) & (1<<7 - 1)
-}
-
 func isCgoType(bi *BinaryInfo, typ godwarf.Type) bool {
 	cu := bi.Images[typ.Common().Index].findCompileUnitForOffset(typ.Common().Offset)
 	if cu == nil {
@@ -2492,7 +2493,7 @@ func (cm constantsMap) Get(typ godwarf.Type) *constantType {
 		sort.Sort(constantValuesByValue(ctyp.values))
 		for i := range ctyp.values {
 			ctyp.values[i].name = strings.TrimPrefix(ctyp.values[i].name, typepkg)
-			if popcnt(uint64(ctyp.values[i].value)) == 1 {
+			if bits.OnesCount64(uint64(ctyp.values[i].value)) == 1 {
 				ctyp.values[i].singleBit = true
 			}
 		}
