@@ -6,7 +6,6 @@ import (
 	"go/parser"
 	"go/token"
 	"io"
-	"io/ioutil"
 	"path/filepath"
 	"reflect"
 	"sort"
@@ -29,7 +28,7 @@ const (
 // Print prints to out a syntax highlighted version of the text read from
 // reader, between lines startLine and endLine.
 func Print(out io.Writer, path string, reader io.Reader, startLine, endLine, arrowLine int, colorEscapes map[Style]string, altTabStr string) error {
-	buf, err := ioutil.ReadAll(reader)
+	buf, err := io.ReadAll(reader)
 	if err != nil {
 		return err
 	}
@@ -134,13 +133,13 @@ func Print(out io.Writer, path string, reader io.Reader, startLine, endLine, arr
 
 		tokposval := nval.FieldByName("TokPos")
 		tokval := nval.FieldByName("Tok")
-		if tokposval != (reflect.Value{}) && tokval != (reflect.Value{}) {
+		if tokposval.IsValid() && tokval.IsValid() {
 			emit(tokval.Interface().(token.Token), tokposval.Interface().(token.Pos), token.NoPos)
 		}
 
 		for _, kwname := range []string{"Case", "Begin", "Defer", "For", "Func", "Go", "Interface", "Map", "Return", "Select", "Struct", "Switch"} {
 			kwposval := nval.FieldByName(kwname)
-			if kwposval != (reflect.Value{}) {
+			if kwposval.IsValid() {
 				kwpos, ok := kwposval.Interface().(token.Pos)
 				if ok && kwpos != token.NoPos {
 					emit(token.ILLEGAL, kwpos, token.NoPos)
