@@ -12,7 +12,6 @@ import (
 	"go/parser"
 	"go/scanner"
 	"io"
-	"io/ioutil"
 	"math"
 	"os"
 	"os/exec"
@@ -228,7 +227,7 @@ If called with the locspec argument it will delete all the breakpoints matching
 toggle <breakpoint name or id>`},
 		{aliases: []string{"goroutines", "grs"}, group: goroutineCmds, cmdFn: c.goroutines, helpMsg: `List program goroutines.
 
-	goroutines [-u|-r|-g|-s] [-t [depth]] [-l] [-with loc expr] [-without loc expr] [-group argument] [-exec command]
+	goroutines [-u|-r|-g|-s] [-t [depth]] [-l] [-with loc expr] [-without loc expr] [-group argument] [-chan expr] [-exec command]
 
 Print out info for every goroutine. The flag controls what information is shown along with each goroutine:
 
@@ -279,6 +278,14 @@ To only display user (or runtime) goroutines, use:
 	goroutines -with user
 	goroutines -without user
 
+CHANNELS
+	
+To only show goroutines waiting to send to or receive from a specific channel use:
+
+	goroutines -chan expr
+	
+Note that 'expr' must not contain spaces.
+
 GROUPING
 
 	goroutines -group (userloc|curloc|goloc|startloc|running|user)
@@ -318,7 +325,7 @@ Called with more arguments it will execute a command on the specified goroutine.
 	breakpoints [-a]
 
 Specifying -a prints all physical breakpoint, including internal breakpoints.`},
-		{aliases: []string{"print", "p"}, group: dataCmds, allowedPrefixes: onPrefix | deferredPrefix, cmdFn: printVar, helpMsg: `Evaluate an expression.
+		{aliases: []string{"print", "p"}, group: dataCmds, allowedPrefixes: onPrefix | deferredPrefix, cmdFn: c.printVar, helpMsg: `Evaluate an expression.
 
 	[goroutine <n>] [frame <m>] print [%format] <expression>
 
@@ -348,6 +355,11 @@ If regex is specified only the functions matching it will be returned.`},
 	types [<regex>]
 
 If regex is specified only the types matching it will be returned.`},
+		{aliases: []string{"packages"}, cmdFn: packages, helpMsg: `Print list of packages.
+
+	packages [<regex>]
+
+If regex is specified only the packages matching it will be returned.`},
 		{aliases: []string{"args"}, allowedPrefixes: onPrefix | deferredPrefix, group: dataCmds, cmdFn: args, helpMsg: `Print function arguments.
 
 	[goroutine <n>] [frame <m>] args [-v] [<regex>]
@@ -902,7 +914,7 @@ func (c *Commands) goroutines(t *Term, ctx callContext, argstr string) error {
 			fmt.Fprintf(t.stdout, "interrupted\n")
 			return nil
 		}
-		gs, groups, start, tooManyGroups, err = t.client.ListGoroutinesWithFilter(start, batchSize, filters, &group)
+		gs, groups, start, tooManyGroups, err = t.client.ListGoroutinesWithFilter(start, batchSize, filters, &group, &api.EvalScope{GoroutineID: -1, Frame: c.frame})
 		if err != nil {
 			return err
 		}
@@ -1634,7 +1646,7 @@ func clearAll(t *Term, ctx callContext, args string) error {
 
 	var locPCs map[uint64]struct{}
 	if args != "" {
-		locs, err := t.client.FindLocation(api.EvalScope{GoroutineID: -1, Frame: 0}, args, true, t.substitutePathRules())
+		locs, _, err := t.client.FindLocation(api.EvalScope{GoroutineID: -1, Frame: 0}, args, true, t.substitutePathRules())
 		if err != nil {
 			return err
 		}
@@ -1702,8 +1714,15 @@ func breakpoints(t *Term, ctx callContext, args string) error {
 		enabled := "(enabled)"
 		if bp.Disabled {
 			enabled = "(disabled)"
+		} else if bp.ExprString != "" {
+			enabled = "(suspended)"
+		}
+		fmt.Fprintf(t.stdout, "%s %s", formatBreakpointName(bp, true), enabled)
+		if bp.ExprString != "" {
+			fmt.Fprintf(t.stdout, " at %s\n", bp.ExprString)
+		} else {
+			fmt.Fprintf(t.stdout, " at %v (%d)\n", t.formatBreakpointLocation(bp), bp.TotalHitCount)
 		}
-		fmt.Fprintf(t.stdout, "%s %s at %v (%d)\n", formatBreakpointName(bp, true), enabled, t.formatBreakpointLocation(bp), bp.TotalHitCount)
 
 		attrs := formatBreakpointAttrs("\t", bp, false)
 
@@ -1783,14 +1802,16 @@ func setBreakpoint(t *Term, ctx callContext, tracepoint bool, argstr string) ([]
 	}
 
 	requestedBp.Tracepoint = tracepoint
-	locs, findLocErr := t.client.FindLocation(ctx.Scope, spec, true, t.substitutePathRules())
+	locs, substSpec, findLocErr := t.client.FindLocation(ctx.Scope, spec, true, t.substitutePathRules())
 	if findLocErr != nil && requestedBp.Name != "" {
 		requestedBp.Name = ""
 		spec = argstr
 		var err2 error
-		locs, err2 = t.client.FindLocation(ctx.Scope, spec, true, t.substitutePathRules())
+		var substSpec2 string
+		locs, substSpec2, err2 = t.client.FindLocation(ctx.Scope, spec, true, t.substitutePathRules())
 		if err2 == nil {
 			findLocErr = nil
+			substSpec = substSpec2
 		}
 	}
 	if findLocErr != nil && shouldAskToSuspendBreakpoint(t) {
@@ -1817,6 +1838,9 @@ func setBreakpoint(t *Term, ctx callContext, tracepoint bool, argstr string) ([]
 	if findLocErr != nil {
 		return nil, findLocErr
 	}
+	if substSpec != "" {
+		spec = substSpec
+	}
 
 	created := []*api.Breakpoint{}
 	for _, loc := range locs {
@@ -1889,14 +1913,21 @@ func tracepoint(t *Term, ctx callContext, args string) error {
 	return err
 }
 
-func runEditor(args ...string) error {
+func getEditorName() (string, error) {
 	var editor string
 	if editor = os.Getenv("DELVE_EDITOR"); editor == "" {
 		if editor = os.Getenv("EDITOR"); editor == "" {
-			return fmt.Errorf("Neither DELVE_EDITOR or EDITOR is set")
+			return "", fmt.Errorf("Neither DELVE_EDITOR or EDITOR is set")
 		}
 	}
+	return editor, nil
+}
 
+func runEditor(args ...string) error {
+	editor, err := getEditorName()
+	if err != nil {
+		return err
+	}
 	cmd := exec.Command(editor, args...)
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
@@ -1909,7 +1940,20 @@ func edit(t *Term, ctx callContext, args string) error {
 	if err != nil {
 		return err
 	}
-	return runEditor(fmt.Sprintf("+%d", lineno), file)
+	editor, err := getEditorName()
+	if err != nil {
+		return err
+	}
+	switch editor {
+	case "code":
+		return runEditor("--goto", fmt.Sprintf("%s:%d", file, lineno))
+	case "hx":
+		return runEditor(fmt.Sprintf("%s:%d", file, lineno))
+	case "vi", "vim", "nvim":
+		return runEditor(fmt.Sprintf("+%d", lineno), file)
+	default:
+		return runEditor(fmt.Sprintf("+%d", lineno), file)
+	}
 }
 
 func watchpoint(t *Term, ctx callContext, args string) error {
@@ -2078,7 +2122,9 @@ func parseFormatArg(args string) (fmtstr, argsOut string) {
 	return v[0], v[1]
 }
 
-func printVar(t *Term, ctx callContext, args string) error {
+const maxPrintVarChanGoroutines = 100
+
+func (c *Commands) printVar(t *Term, ctx callContext, args string) error {
 	if len(args) == 0 {
 		return fmt.Errorf("not enough arguments")
 	}
@@ -2092,7 +2138,25 @@ func printVar(t *Term, ctx callContext, args string) error {
 		return err
 	}
 
+	t.stdout.pw.PageMaybe(nil)
+
 	fmt.Fprintln(t.stdout, val.MultilineString("", fmtstr))
+
+	if val.Kind == reflect.Chan {
+		fmt.Fprintln(t.stdout)
+		gs, _, _, _, err := t.client.ListGoroutinesWithFilter(0, maxPrintVarChanGoroutines, []api.ListGoroutinesFilter{{Kind: api.GoroutineWaitingOnChannel, Arg: fmt.Sprintf("*(*%q)(%#x)", val.Type, val.Addr)}}, nil, &ctx.Scope)
+		if err != nil {
+			fmt.Fprintf(t.stdout, "Error reading channel wait queue: %v", err)
+		} else {
+			fmt.Fprintln(t.stdout, "Goroutines waiting on this channel:")
+			state, err := t.client.GetState()
+			if err != nil {
+				fmt.Fprintf(t.stdout, "Error printing channel wait queue: %v", err)
+			}
+			var done bool
+			c.printGoroutines(t, ctx, "", gs, api.FglUserCurrent, 0, 0, "", &done, state)
+		}
+	}
 	return nil
 }
 
@@ -2146,6 +2210,7 @@ func (t *Term) printFilteredVariables(varType string, vars []api.Variable, filte
 		return err
 	}
 	match := false
+	t.stdout.pw.PageMaybe(nil)
 	for _, v := range vars {
 		if reg == nil || reg.Match([]byte(v.Name)) {
 			match = true
@@ -2186,6 +2251,18 @@ func sources(t *Term, ctx callContext, args string) error {
 	return t.printSortedStrings(t.client.ListSources(args))
 }
 
+func packages(t *Term, ctx callContext, args string) error {
+	info, err := t.client.ListPackagesBuildInfo(args, false)
+	if err != nil {
+		return err
+	}
+	pkgs := make([]string, 0, len(info))
+	for _, i := range info {
+		pkgs = append(pkgs, i.ImportPath)
+	}
+	return t.printSortedStrings(pkgs, nil)
+}
+
 func funcs(t *Term, ctx callContext, args string) error {
 	return t.printSortedStrings(t.client.ListFunctions(args))
 }
@@ -2424,7 +2501,7 @@ func getLocation(t *Term, ctx callContext, args string, showContext bool) (file
 		return loc.File, loc.Line, true, nil
 
 	default:
-		locs, err := t.client.FindLocation(ctx.Scope, args, false, t.substitutePathRules())
+		locs, _, err := t.client.FindLocation(ctx.Scope, args, false, t.substitutePathRules())
 		if err != nil {
 			return "", 0, false, err
 		}
@@ -2498,7 +2575,7 @@ func disassCommand(t *Term, ctx callContext, args string) error {
 
 	switch cmd {
 	case "":
-		locs, err := t.client.FindLocation(ctx.Scope, "+0", true, t.substitutePathRules())
+		locs, _, err := t.client.FindLocation(ctx.Scope, "+0", true, t.substitutePathRules())
 		if err != nil {
 			return err
 		}
@@ -2518,7 +2595,7 @@ func disassCommand(t *Term, ctx callContext, args string) error {
 		}
 		disasm, disasmErr = t.client.DisassembleRange(ctx.Scope, uint64(startpc), uint64(endpc), flavor)
 	case "-l":
-		locs, err := t.client.FindLocation(ctx.Scope, rest, true, t.substitutePathRules())
+		locs, _, err := t.client.FindLocation(ctx.Scope, rest, true, t.substitutePathRules())
 		if err != nil {
 			return err
 		}
@@ -2548,7 +2625,7 @@ func libraries(t *Term, ctx callContext, args string) error {
 	for i := range libs {
 		fmt.Fprintf(t.stdout, "%"+strconv.Itoa(d)+"d. %#x %s\n", i, libs[i].Address, libs[i].Path)
 		if libs[i].LoadError != "" {
-			fmt.Fprintf(t.stdout, "    Load error: %s", libs[i].LoadError)
+			fmt.Fprintf(t.stdout, "    Load error: %s\n", libs[i].LoadError)
 		}
 	}
 	return nil
@@ -2562,7 +2639,7 @@ func digits(n int) int {
 }
 
 func printStack(t *Term, out io.Writer, stack []api.Stackframe, ind string, offsets bool) {
-	api.PrintStack(t.formatPath, out, stack, ind, offsets, func(api.Stackframe) bool { return true })
+	api.PrintStack(t.formatPath, out, stack, ind, offsets, t.stackTraceColors, func(api.Stackframe) bool { return true })
 }
 
 func printcontext(t *Term, state *api.DebuggerState) {
@@ -2933,7 +3010,7 @@ func (c *Commands) onCmd(t *Term, ctx callContext, argstr string) error {
 	ctx.Breakpoint = bp
 
 	if args[1] == "-edit" {
-		f, err := ioutil.TempFile("", "dlv-on-cmd-")
+		f, err := os.CreateTemp("", "dlv-on-cmd-")
 		if err != nil {
 			return err
 		}
@@ -3277,7 +3354,12 @@ func target(t *Term, ctx callContext, args string) error {
 		return nil
 	case "follow-exec":
 		if len(argv) == 1 {
-			return errors.New("not enough arguments")
+			if t.client.FollowExecEnabled() {
+				fmt.Fprintf(t.stdout, "Follow exec is enabled.\n")
+			} else {
+				fmt.Fprintf(t.stdout, "Follow exec is disabled.\n")
+			}
+			return nil
 		}
 		argv = config.Split2PartsBySpace(argv[1])
 		switch argv[0] {
@@ -3316,6 +3398,8 @@ func target(t *Term, ctx callContext, args string) error {
 			return fmt.Errorf("could not find target %d", pid)
 		}
 		return nil
+	case "":
+		return errors.New("not enough arguments for 'target'")
 	default:
 		return fmt.Errorf("unknown command 'target %s'", argv[0])
 	}
