@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"flag"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"os"
@@ -122,7 +121,7 @@ func withTestTerminalBuildFlags(name string, t testing.TB, buildFlags test.Build
 	if testBackend == "rr" {
 		test.MustHaveRecordingAllowed(t)
 	}
-	os.Setenv("TERM", "dumb")
+	t.Setenv("TERM", "dumb")
 	listener, err := net.Listen("tcp", "127.0.0.1:0")
 	if err != nil {
 		t.Fatalf("couldn't start listener: %s\n", err)
@@ -305,6 +304,9 @@ func TestExitStatus(t *testing.T) {
 }
 
 func TestScopePrefix(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" && buildMode == "pie" {
+		t.Skip("pie mode broken on ppc64le")
+	}
 	const goroutinesLinePrefix = "  Goroutine "
 	const goroutinesCurLinePrefix = "* Goroutine "
 	test.AllowRecording(t)
@@ -873,6 +875,9 @@ func TestIssue1090(t *testing.T) {
 }
 
 func TestPrintContextParkedGoroutine(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" && buildMode == "pie" {
+		t.Skip("pie mode broken on ppc64le")
+	}
 	withTestTerminal("goroutinestackprog", t, func(term *FakeTerminal) {
 		term.MustExec("break stacktraceme")
 		term.MustExec("continue")
@@ -946,6 +951,9 @@ func TestOptimizationCheck(t *testing.T) {
 }
 
 func TestTruncateStacktrace(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" && buildMode == "pie" {
+		t.Skip("pie mode broken on ppc64le")
+	}
 	const stacktraceTruncatedMessage = "(truncated)"
 	withTestTerminal("stacktraceprog", t, func(term *FakeTerminal) {
 		term.MustExec("break main.stacktraceme")
@@ -966,6 +974,9 @@ func TestTruncateStacktrace(t *testing.T) {
 func TestIssue1493(t *testing.T) {
 	// The 'regs' command without the '-a' option should only return
 	// general purpose registers.
+	if runtime.GOARCH == "ppc64le" {
+		t.Skip("skipping, some registers such as vector registers are currently not loaded")
+	}
 	withTestTerminal("continuetestprog", t, func(term *FakeTerminal) {
 		r := term.MustExec("regs")
 		nr := len(strings.Split(r, "\n"))
@@ -984,6 +995,9 @@ func findStarFile(name string) string {
 }
 
 func TestIssue1598(t *testing.T) {
+	if buildMode == "pie" && runtime.GOARCH == "ppc64le" {
+		t.Skip("Debug function call Test broken in PIE mode")
+	}
 	test.MustSupportFunctionCalls(t, testBackend)
 	withTestTerminal("issue1598", t, func(term *FakeTerminal) {
 		term.MustExec("break issue1598.go:5")
@@ -1228,6 +1242,19 @@ func TestHitCondBreakpoint(t *testing.T) {
 	})
 }
 
+func TestCondBreakpointWithFrame(t *testing.T) {
+	withTestTerminal("condframe", t, func(term *FakeTerminal) {
+		term.MustExec("break bp1 callme2")
+		term.MustExec("condition bp1 runtime.frame(1).i == 3")
+		term.MustExec("continue")
+		out := term.MustExec("frame 1 print i")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "3\n") {
+			t.Fatalf("wrong value of i")
+		}
+	})
+}
+
 func TestClearCondBreakpoint(t *testing.T) {
 	withTestTerminal("break", t, func(term *FakeTerminal) {
 		term.MustExec("break main.main:4")
@@ -1342,16 +1369,16 @@ func TestTranscript(t *testing.T) {
 		if !strings.HasPrefix(out, "> main.main()") {
 			t.Fatalf("Wrong output for next: <%s>", out)
 		}
-		fh, err := ioutil.TempFile("", "test-transcript-*")
+		fh, err := os.CreateTemp("", "test-transcript-*")
 		if err != nil {
-			t.Fatalf("TempFile: %v", err)
+			t.Fatalf("CreateTemp: %v", err)
 		}
 		name := fh.Name()
 		fh.Close()
 		t.Logf("output to %q", name)
 
 		slurp := func() string {
-			b, err := ioutil.ReadFile(name)
+			b, err := os.ReadFile(name)
 			if err != nil {
 				t.Fatalf("could not read transcript file: %v", err)
 			}
@@ -1381,6 +1408,9 @@ func TestTranscript(t *testing.T) {
 }
 
 func TestDisassPosCmd(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" && buildMode == "pie" {
+		t.Skip("pie mode broken on ppc64le")
+	}
 	withTestTerminal("testvariables2", t, func(term *FakeTerminal) {
 		term.MustExec("continue")
 		out := term.MustExec("step-instruction")
@@ -1404,3 +1434,104 @@ func TestCreateBreakpointByLocExpr(t *testing.T) {
 		}
 	})
 }
+
+func TestRestartBreakpoints(t *testing.T) {
+	// Tests that breakpoints set using just a line number and with a line
+	// offset are preserved after restart. See issue #3423.
+	withTestTerminal("continuetestprog", t, func(term *FakeTerminal) {
+		term.MustExec("break main.main")
+		term.MustExec("continue")
+		term.MustExec("break 9")
+		term.MustExec("break +1")
+		out := term.MustExec("breakpoints")
+		t.Log("breakpoints before:\n", out)
+		term.MustExec("restart")
+		out = term.MustExec("breakpoints")
+		t.Log("breakpoints after:\n", out)
+		bps, err := term.client.ListBreakpoints(false)
+		assertNoError(t, err, "ListBreakpoints")
+		for _, bp := range bps {
+			if bp.ID < 0 {
+				continue
+			}
+			if bp.Addr == 0 {
+				t.Fatalf("breakpoint %d has address 0", bp.ID)
+			}
+		}
+	})
+}
+
+func TestListPackages(t *testing.T) {
+	test.AllowRecording(t)
+	withTestTerminal("goroutinestackprog", t, func(term *FakeTerminal) {
+		out := term.MustExec("packages")
+		t.Logf("> packages\n%s", out)
+		seen := map[string]bool{}
+		for _, p := range strings.Split(strings.TrimSpace(out), "\n") {
+			seen[p] = true
+		}
+		if !seen["main"] || !seen["runtime"] {
+			t.Error("output omits 'main' and 'runtime'")
+		}
+
+		out = term.MustExec("packages runtime")
+		t.Logf("> packages runtime\n%s", out)
+
+		for _, p := range strings.Split(strings.TrimSpace(out), "\n") {
+			if !strings.Contains(p, "runtime") {
+				t.Errorf("output includes unexpected %q", p)
+			}
+			seen[p] = true
+		}
+		if !seen["runtime"] {
+			t.Error("output omits 'runtime'")
+		}
+	})
+}
+
+func TestSubstitutePathAndList(t *testing.T) {
+	// checks that substitute path rules do not remain cached after a -clear.
+	// See issue #3565.
+	if runtime.GOOS == "windows" {
+		t.Skip("test is not valid on windows due to path separators")
+	}
+	withTestTerminal("math", t, func(term *FakeTerminal) {
+		term.MustExec("break main.main")
+		term.MustExec("continue")
+		fixturesDir, _ := filepath.Abs(test.FindFixturesDir())
+		term.MustExec("config substitute-path " + fixturesDir + " /blah")
+		out, _ := term.Exec("list")
+		t.Logf("list output %s", out)
+		if !strings.Contains(out, "/blah/math.go") {
+			t.Fatalf("bad output")
+		}
+		term.MustExec("config substitute-path -clear")
+		term.MustExec("config substitute-path " + fixturesDir + " /blah2")
+		out, _ = term.Exec("list")
+		t.Logf("list output %s", out)
+		if !strings.Contains(out, "/blah2/math.go") {
+			t.Fatalf("bad output")
+		}
+	})
+}
+
+func TestDisplay(t *testing.T) {
+	// Tests that display command works. See issue #3595.
+	type testCase struct{ in, tgt string }
+	withTestTerminal("testvariables2", t, func(term *FakeTerminal) {
+		term.MustExec("continue")
+
+		for _, tc := range []testCase{
+			{"string(byteslice)", `0: string(byteslice) = "tèst"`},
+			{"string(byteslice[1:])", `0: string(byteslice[1:]) = "èst"`},
+			{"%s string(byteslice)", `0: string(byteslice) = tèst`},
+		} {
+			out := term.MustExec("display -a " + tc.in)
+			t.Logf("%q -> %q", tc.in, out)
+			if !strings.Contains(out, tc.tgt) {
+				t.Errorf("wrong output for 'display -a %s':\n\tgot: %q\n\texpected: %q", tc.in, out, tc.tgt)
+			}
+			term.MustExec("display -d 0")
+		}
+	})
+}
