@@ -18,36 +18,73 @@ const (
 	indentString = "\t"
 )
 
+type prettyFlags uint8
+
+const (
+	prettyTop prettyFlags = 1 << iota
+	prettyNewlines
+	prettyIncludeType
+	prettyShortenType
+)
+
+func (flags prettyFlags) top() bool         { return flags&prettyTop != 0 }
+func (flags prettyFlags) includeType() bool { return flags&prettyIncludeType != 0 }
+func (flags prettyFlags) newlines() bool    { return flags&prettyNewlines != 0 }
+func (flags prettyFlags) shortenType() bool { return flags&prettyShortenType != 0 }
+
+func (flags prettyFlags) set(flag prettyFlags, v bool) prettyFlags {
+	if v {
+		return flags | flag
+	} else {
+		return flags &^ flag
+	}
+}
+
 // SinglelineString returns a representation of v on a single line.
 func (v *Variable) SinglelineString() string {
 	var buf bytes.Buffer
-	v.writeTo(&buf, true, false, true, "", "")
+	v.writeTo(&buf, prettyTop|prettyIncludeType, "", "")
+	return buf.String()
+}
+
+// SinglelineStringWithShortTypes returns a representation of v on a single line, with types shortened.
+func (v *Variable) SinglelineStringWithShortTypes() string {
+	var buf bytes.Buffer
+	v.writeTo(&buf, prettyTop|prettyIncludeType|prettyShortenType, "", "")
 	return buf.String()
 }
 
 // SinglelineStringFormatted returns a representation of v on a single line, using the format specified by fmtstr.
 func (v *Variable) SinglelineStringFormatted(fmtstr string) string {
 	var buf bytes.Buffer
-	v.writeTo(&buf, true, false, true, "", fmtstr)
+	v.writeTo(&buf, prettyTop|prettyIncludeType, "", fmtstr)
 	return buf.String()
 }
 
 // MultilineString returns a representation of v on multiple lines.
 func (v *Variable) MultilineString(indent, fmtstr string) string {
 	var buf bytes.Buffer
-	v.writeTo(&buf, true, true, true, indent, fmtstr)
+	v.writeTo(&buf, prettyTop|prettyNewlines|prettyIncludeType, indent, fmtstr)
 	return buf.String()
 }
 
-func (v *Variable) writeTo(buf io.Writer, top, newlines, includeType bool, indent, fmtstr string) {
+func (v *Variable) typeStr(flags prettyFlags) string {
+	if flags.shortenType() {
+		return ShortenType(v.Type)
+	}
+
+	return v.Type
+}
+
+func (v *Variable) writeTo(buf io.Writer, flags prettyFlags, indent, fmtstr string) {
 	if v.Unreadable != "" {
 		fmt.Fprintf(buf, "(unreadable %s)", v.Unreadable)
 		return
 	}
 
-	if !top && v.Addr == 0 && v.Value == "" {
-		if includeType && v.Type != "void" {
-			fmt.Fprintf(buf, "%s nil", v.Type)
+	if !flags.top() && v.Addr == 0 && v.Value == "" {
+		if flags.includeType() && v.Type != "void" {
+			fmt.Fprintf(buf, "%s nil", v.typeStr(flags))
 		} else {
 			fmt.Fprint(buf, "nil")
 		}
@@ -56,21 +93,21 @@ func (v *Variable) writeTo(buf io.Writer, top, newlines, includeType bool, inden
 
 	switch v.Kind {
 	case reflect.Slice:
-		v.writeSliceTo(buf, newlines, includeType, indent, fmtstr)
+		v.writeSliceTo(buf, flags, indent, fmtstr)
 	case reflect.Array:
-		v.writeArrayTo(buf, newlines, includeType, indent, fmtstr)
+		v.writeArrayTo(buf, flags, indent, fmtstr)
 	case reflect.Ptr:
 		if v.Type == "" || len(v.Children) == 0 {
 			fmt.Fprint(buf, "nil")
 		} else if v.Children[0].OnlyAddr && v.Children[0].Addr != 0 {
-			if strings.Contains(v.Type, "/") {
-				fmt.Fprintf(buf, "(%q)(%#x)", v.Type, v.Children[0].Addr)
-			} else {
-				fmt.Fprintf(buf, "(%s)(%#x)", v.Type, v.Children[0].Addr)
-			}
+			v.writePointerTo(buf, flags)
 		} else {
+			if flags.top() && flags.newlines() && v.Children[0].Addr != 0 {
+				v.writePointerTo(buf, flags)
+				fmt.Fprint(buf, "\n")
+			}
 			fmt.Fprint(buf, "*")
-			v.Children[0].writeTo(buf, false, newlines, includeType, indent, fmtstr)
+			v.Children[0].writeTo(buf, flags.set(prettyTop, false), indent, fmtstr)
 		}
 	case reflect.UnsafePointer:
 		if len(v.Children) == 0 {
@@ -79,17 +116,21 @@ func (v *Variable) writeTo(buf io.Writer, top, newlines, includeType bool, inden
 			fmt.Fprintf(buf, "unsafe.Pointer(%#x)", v.Children[0].Addr)
 		}
 	case reflect.Chan:
-		if newlines {
-			v.writeStructTo(buf, newlines, includeType, indent, fmtstr)
+		if flags.newlines() {
+			v.writeStructTo(buf, flags, indent, fmtstr)
 		} else {
 			if len(v.Children) == 0 {
-				fmt.Fprintf(buf, "%s nil", v.Type)
+				fmt.Fprintf(buf, "%s nil", v.typeStr(flags))
 			} else {
-				fmt.Fprintf(buf, "%s %s/%s", v.Type, v.Children[0].Value, v.Children[1].Value)
+				fmt.Fprintf(buf, "%s %s/%s", v.typeStr(flags), v.Children[0].Value, v.Children[1].Value)
 			}
 		}
 	case reflect.Struct:
-		v.writeStructTo(buf, newlines, includeType, indent, fmtstr)
+		if v.Value != "" {
+			fmt.Fprintf(buf, "%s(%s)", v.typeStr(flags), v.Value)
+			flags = flags.set(prettyIncludeType, false)
+		}
+		v.writeStructTo(buf, flags, indent, fmtstr)
 	case reflect.Interface:
 		if v.Addr == 0 {
 			// an escaped interface variable that points to nil, this shouldn't
@@ -97,15 +138,15 @@ func (v *Variable) writeTo(buf io.Writer, top, newlines, includeType bool, inden
 			fmt.Fprintf(buf, "nil")
 			return
 		}
-		if includeType {
+		if flags.includeType() {
 			if v.Children[0].Kind == reflect.Invalid {
-				fmt.Fprintf(buf, "%s ", v.Type)
+				fmt.Fprintf(buf, "%s ", v.typeStr(flags))
 				if v.Children[0].Addr == 0 {
 					fmt.Fprint(buf, "nil")
 					return
 				}
 			} else {
-				fmt.Fprintf(buf, "%s(%s) ", v.Type, v.Children[0].Type)
+				fmt.Fprintf(buf, "%s(%s) ", v.typeStr(flags), v.Children[0].Type)
 			}
 		}
 		data := v.Children[0]
@@ -117,19 +158,19 @@ func (v *Variable) writeTo(buf io.Writer, top, newlines, includeType bool, inden
 			} else if data.Children[0].OnlyAddr {
 				fmt.Fprintf(buf, "0x%x", v.Children[0].Addr)
 			} else {
-				v.Children[0].writeTo(buf, false, newlines, !includeType, indent, fmtstr)
+				v.Children[0].writeTo(buf, flags.set(prettyTop, false).set(prettyIncludeType, !flags.includeType()), indent, fmtstr)
 			}
 		} else if data.OnlyAddr {
 			if strings.Contains(v.Type, "/") {
-				fmt.Fprintf(buf, "*(*%q)(%#x)", v.Type, v.Addr)
+				fmt.Fprintf(buf, "*(*%q)(%#x)", v.typeStr(flags), v.Addr)
 			} else {
-				fmt.Fprintf(buf, "*(*%s)(%#x)", v.Type, v.Addr)
+				fmt.Fprintf(buf, "*(*%s)(%#x)", v.typeStr(flags), v.Addr)
 			}
 		} else {
-			v.Children[0].writeTo(buf, false, newlines, !includeType, indent, fmtstr)
+			v.Children[0].writeTo(buf, flags.set(prettyTop, false).set(prettyIncludeType, !flags.includeType()), indent, fmtstr)
 		}
 	case reflect.Map:
-		v.writeMapTo(buf, newlines, includeType, indent, fmtstr)
+		v.writeMapTo(buf, flags, indent, fmtstr)
 	case reflect.Func:
 		if v.Value == "" {
 			fmt.Fprint(buf, "nil")
@@ -141,6 +182,14 @@ func (v *Variable) writeTo(buf io.Writer, top, newlines, includeType bool, inden
 	}
 }
 
+func (v *Variable) writePointerTo(buf io.Writer, flags prettyFlags) {
+	if strings.Contains(v.Type, "/") {
+		fmt.Fprintf(buf, "(%q)(%#x)", v.typeStr(flags), v.Children[0].Addr)
+	} else {
+		fmt.Fprintf(buf, "(%s)(%#x)", v.typeStr(flags), v.Children[0].Addr)
+	}
+}
+
 func (v *Variable) writeBasicType(buf io.Writer, fmtstr string) {
 	if v.Value == "" && v.Kind != reflect.String {
 		fmt.Fprintf(buf, "(unknown %s)", v.Kind)
@@ -161,7 +210,7 @@ func (v *Variable) writeBasicType(buf io.Writer, fmtstr string) {
 			buf.Write([]byte(v.Value))
 			return
 		}
-		n, _ := strconv.ParseInt(v.Value, 10, 64)
+		n, _ := strconv.ParseInt(ExtractIntValue(v.Value), 10, 64)
 		fmt.Fprintf(buf, fmtstr, n)
 
 	case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64, reflect.Uintptr:
@@ -169,7 +218,7 @@ func (v *Variable) writeBasicType(buf io.Writer, fmtstr string) {
 			buf.Write([]byte(v.Value))
 			return
 		}
-		n, _ := strconv.ParseUint(v.Value, 10, 64)
+		n, _ := strconv.ParseUint(ExtractIntValue(v.Value), 10, 64)
 		fmt.Fprintf(buf, fmtstr, n)
 
 	case reflect.Float32, reflect.Float64:
@@ -203,39 +252,50 @@ func (v *Variable) writeBasicType(buf io.Writer, fmtstr string) {
 	}
 }
 
-func (v *Variable) writeSliceTo(buf io.Writer, newlines, includeType bool, indent, fmtstr string) {
-	if includeType {
-		fmt.Fprintf(buf, "%s len: %d, cap: %d, ", v.Type, v.Len, v.Cap)
+func ExtractIntValue(s string) string {
+	if s == "" || s[len(s)-1] != ')' {
+		return s
+	}
+	open := strings.LastIndex(s, "(")
+	if open < 0 {
+		return s
+	}
+	return s[open+1 : len(s)-1]
+}
+
+func (v *Variable) writeSliceTo(buf io.Writer, flags prettyFlags, indent, fmtstr string) {
+	if flags.includeType() {
+		fmt.Fprintf(buf, "%s len: %d, cap: %d, ", v.typeStr(flags), v.Len, v.Cap)
 	}
 	if v.Base == 0 && len(v.Children) == 0 {
 		fmt.Fprintf(buf, "nil")
 		return
 	}
-	v.writeSliceOrArrayTo(buf, newlines, indent, fmtstr)
+	v.writeSliceOrArrayTo(buf, flags, indent, fmtstr)
 }
 
-func (v *Variable) writeArrayTo(buf io.Writer, newlines, includeType bool, indent, fmtstr string) {
-	if includeType {
-		fmt.Fprintf(buf, "%s ", v.Type)
+func (v *Variable) writeArrayTo(buf io.Writer, flags prettyFlags, indent, fmtstr string) {
+	if flags.includeType() {
+		fmt.Fprintf(buf, "%s ", v.typeStr(flags))
 	}
-	v.writeSliceOrArrayTo(buf, newlines, indent, fmtstr)
+	v.writeSliceOrArrayTo(buf, flags, indent, fmtstr)
 }
 
-func (v *Variable) writeStructTo(buf io.Writer, newlines, includeType bool, indent, fmtstr string) {
+func (v *Variable) writeStructTo(buf io.Writer, flags prettyFlags, indent, fmtstr string) {
 	if int(v.Len) != len(v.Children) && len(v.Children) == 0 {
 		if strings.Contains(v.Type, "/") {
-			fmt.Fprintf(buf, "(*%q)(%#x)", v.Type, v.Addr)
+			fmt.Fprintf(buf, "(*%q)(%#x)", v.typeStr(flags), v.Addr)
 		} else {
-			fmt.Fprintf(buf, "(*%s)(%#x)", v.Type, v.Addr)
+			fmt.Fprintf(buf, "(*%s)(%#x)", v.typeStr(flags), v.Addr)
 		}
 		return
 	}
 
-	if includeType {
-		fmt.Fprintf(buf, "%s ", v.Type)
+	if flags.includeType() {
+		fmt.Fprintf(buf, "%s ", v.typeStr(flags))
 	}
 
-	nl := v.shouldNewlineStruct(newlines)
+	nl := v.shouldNewlineStruct(flags.newlines())
 
 	fmt.Fprint(buf, "{")
 
@@ -244,7 +304,7 @@ func (v *Variable) writeStructTo(buf io.Writer, newlines, includeType bool, inde
 			fmt.Fprintf(buf, "\n%s%s", indent, indentString)
 		}
 		fmt.Fprintf(buf, "%s: ", v.Children[i].Name)
-		v.Children[i].writeTo(buf, false, nl, true, indent+indentString, fmtstr)
+		v.Children[i].writeTo(buf, prettyIncludeType.set(prettyNewlines, nl), indent+indentString, fmtstr)
 		if i != len(v.Children)-1 || nl {
 			fmt.Fprint(buf, ",")
 			if !nl {
@@ -265,16 +325,16 @@ func (v *Variable) writeStructTo(buf io.Writer, newlines, includeType bool, inde
 	fmt.Fprint(buf, "}")
 }
 
-func (v *Variable) writeMapTo(buf io.Writer, newlines, includeType bool, indent, fmtstr string) {
-	if includeType {
-		fmt.Fprintf(buf, "%s ", v.Type)
+func (v *Variable) writeMapTo(buf io.Writer, flags prettyFlags, indent, fmtstr string) {
+	if flags.includeType() {
+		fmt.Fprintf(buf, "%s ", v.typeStr(flags))
 	}
 	if v.Base == 0 && len(v.Children) == 0 {
 		fmt.Fprintf(buf, "nil")
 		return
 	}
 
-	nl := newlines && (len(v.Children) > 0)
+	nl := flags.newlines() && (len(v.Children) > 0)
 
 	fmt.Fprint(buf, "[")
 
@@ -286,9 +346,9 @@ func (v *Variable) writeMapTo(buf io.Writer, newlines, includeType bool, indent,
 			fmt.Fprintf(buf, "\n%s%s", indent, indentString)
 		}
 
-		key.writeTo(buf, false, false, false, indent+indentString, fmtstr)
+		key.writeTo(buf, 0, indent+indentString, fmtstr)
 		fmt.Fprint(buf, ": ")
-		value.writeTo(buf, false, nl, false, indent+indentString, fmtstr)
+		value.writeTo(buf, prettyFlags(0).set(prettyNewlines, nl), indent+indentString, fmtstr)
 		if i != len(v.Children)-1 || nl {
 			fmt.Fprint(buf, ", ")
 		}
@@ -380,15 +440,15 @@ func (v *Variable) shouldNewlineStruct(newlines bool) bool {
 	return false
 }
 
-func (v *Variable) writeSliceOrArrayTo(buf io.Writer, newlines bool, indent, fmtstr string) {
-	nl := v.shouldNewlineArray(newlines)
+func (v *Variable) writeSliceOrArrayTo(buf io.Writer, flags prettyFlags, indent, fmtstr string) {
+	nl := v.shouldNewlineArray(flags.newlines())
 	fmt.Fprint(buf, "[")
 
 	for i := range v.Children {
 		if nl {
 			fmt.Fprintf(buf, "\n%s%s", indent, indentString)
 		}
-		v.Children[i].writeTo(buf, false, nl, false, indent+indentString, fmtstr)
+		v.Children[i].writeTo(buf, prettyFlags(0).set(prettyNewlines, nl), indent+indentString, fmtstr)
 		if i != len(v.Children)-1 || nl {
 			fmt.Fprint(buf, ",")
 		}
@@ -429,7 +489,7 @@ func PrettyExamineMemory(address uintptr, memArea []byte, isLittleEndian bool, f
 		addrFmt string
 	)
 
-	// Diffrent versions of golang output differently about '#'.
+	// Different versions of golang output differently about '#'.
 	// See https://ci.appveyor.com/project/derekparker/delve-facy3/builds/30179356.
 	switch format {
 	case 'b':
@@ -445,7 +505,7 @@ func PrettyExamineMemory(address uintptr, memArea []byte, isLittleEndian bool, f
 		cols = 8
 		colFormat = fmt.Sprintf("0x%%0%dx", colBytes*2) // Always keep one leading '0x' for hex.
 	default:
-		return fmt.Sprintf("not supprted format %q\n", string(format))
+		return fmt.Sprintf("not supported format %q\n", string(format))
 	}
 	colFormat += "\t"
 
@@ -504,7 +564,13 @@ func digits(n int) int {
 	return int(math.Floor(math.Log10(float64(n)))) + 1
 }
 
-func PrintStack(formatPath func(string) string, out io.Writer, stack []Stackframe, ind string, offsets bool, include func(Stackframe) bool) {
+type StackTraceColors struct {
+	FunctionColor string
+	BasenameColor string
+	NormalColor   string
+}
+
+func PrintStack(formatPath func(string) string, out io.Writer, stack []Stackframe, ind string, offsets bool, stc StackTraceColors, include func(Stackframe) bool) {
 	if len(stack) == 0 {
 		return
 	}
@@ -517,8 +583,22 @@ func PrintStack(formatPath func(string) string, out io.Writer, stack []Stackfram
 		extranl = extranl || (len(stack[i].Defers) > 0) || (len(stack[i].Arguments) > 0) || (len(stack[i].Locals) > 0)
 	}
 
+	fileLine := func(file string, line int) string {
+		file = formatPath(file)
+		if stc.BasenameColor == "" {
+			return fmt.Sprintf("%s:%d", file, line)
+		}
+		slash := strings.LastIndex(file, "/")
+		if slash < 0 {
+			slash = 0
+		} else {
+			slash++
+		}
+		return fmt.Sprintf("%s%s%s:%d%s", file[:slash], stc.BasenameColor, file[slash:], line, stc.NormalColor)
+	}
+
 	d := digits(len(stack) - 1)
-	fmtstr := "%s%" + strconv.Itoa(d) + "d  0x%016x in %s\n"
+	fmtstr := "%s%" + strconv.Itoa(d) + "d  0x%016x in " + stc.FunctionColor + "%s" + stc.NormalColor + "\n"
 	s := ind + strings.Repeat(" ", d+2+len(ind))
 
 	for i := range stack {
@@ -530,7 +610,7 @@ func PrintStack(formatPath func(string) string, out io.Writer, stack []Stackfram
 			continue
 		}
 		fmt.Fprintf(out, fmtstr, ind, i, stack[i].PC, stack[i].Function.Name())
-		fmt.Fprintf(out, "%sat %s:%d\n", s, formatPath(stack[i].File), stack[i].Line)
+		fmt.Fprintf(out, "%sat %s\n", s, fileLine(stack[i].File, stack[i].Line))
 
 		if offsets {
 			fmt.Fprintf(out, "%sframe: %+#x frame pointer %+#x\n", s, stack[i].FrameOffset, stack[i].FramePointerOffset)
@@ -543,9 +623,9 @@ func PrintStack(formatPath func(string) string, out io.Writer, stack []Stackfram
 				fmt.Fprintf(out, "%s(unreadable defer: %s)\n", deferHeader, d.Unreadable)
 				continue
 			}
-			fmt.Fprintf(out, "%s%#016x in %s\n", deferHeader, d.DeferredLoc.PC, d.DeferredLoc.Function.Name())
+			fmt.Fprintf(out, "%s%#016x in %s%s%s\n", deferHeader, d.DeferredLoc.PC, stc.FunctionColor, d.DeferredLoc.Function.Name(), stc.NormalColor)
 			fmt.Fprintf(out, "%sat %s:%d\n", s2, formatPath(d.DeferredLoc.File), d.DeferredLoc.Line)
-			fmt.Fprintf(out, "%sdeferred by %s at %s:%d\n", s2, d.DeferLoc.Function.Name(), formatPath(d.DeferLoc.File), d.DeferLoc.Line)
+			fmt.Fprintf(out, "%sdeferred by %s at %s\n", s2, d.DeferLoc.Function.Name(), fileLine(d.DeferLoc.File, d.DeferLoc.Line))
 		}
 
 		for j := range stack[i].Arguments {
