@@ -3,6 +3,7 @@ package rpc2
 import (
 	"errors"
 	"fmt"
+	"regexp"
 	"sort"
 	"time"
 
@@ -464,9 +465,9 @@ func (s *RPCServer) ListRegisters(arg ListRegistersIn, out *ListRegistersOut) er
 	var err error
 
 	if arg.Scope != nil {
-		regs, err = s.debugger.ScopeRegisters(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.IncludeFp)
+		regs, err = s.debugger.ScopeRegisters(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall)
 	} else {
-		regs, err = s.debugger.ThreadRegisters(arg.ThreadID, arg.IncludeFp)
+		regs, err = s.debugger.ThreadRegisters(arg.ThreadID)
 	}
 	if err != nil {
 		return err
@@ -534,7 +535,8 @@ func (s *RPCServer) Eval(arg EvalIn, out *EvalOut) error {
 	if cfg == nil {
 		cfg = &api.LoadConfig{FollowPointers: true, MaxVariableRecurse: 1, MaxStringLen: 64, MaxArrayValues: 64, MaxStructFields: -1}
 	}
-	v, err := s.debugger.EvalVariableInScope(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Expr, *api.LoadConfigToProc(cfg))
+	pcfg := *api.LoadConfigToProc(cfg)
+	v, err := s.debugger.EvalVariableInScope(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Expr, pcfg)
 	if err != nil {
 		return err
 	}
@@ -617,6 +619,8 @@ type ListGoroutinesIn struct {
 
 	Filters []api.ListGoroutinesFilter
 	api.GoroutineGroupingOptions
+
+	EvalScope *api.EvalScope
 }
 
 type ListGoroutinesOut struct {
@@ -663,7 +667,37 @@ func (s *RPCServer) ListGoroutines(arg ListGoroutinesIn, out *ListGoroutinesOut)
 	//TODO(aarzilli): if arg contains a running goroutines filter (not negated)
 	// and start == 0 and count == 0 then we can optimize this by just looking
 	// at threads directly.
-	gs, nextg, err := s.debugger.Goroutines(arg.Start, arg.Count)
+
+	var gs []*proc.G
+	var nextg int
+	var err error
+	var gsLoaded bool
+
+	for _, filter := range arg.Filters {
+		if filter.Kind == api.GoroutineWaitingOnChannel {
+			if filter.Negated {
+				return errors.New("channel filter can not be negated")
+			}
+			if arg.Count == 0 {
+				return errors.New("count == 0 not allowed with a channel filter")
+			}
+			if arg.EvalScope == nil {
+				return errors.New("channel filter without eval scope")
+			}
+			gs, err = s.debugger.ChanGoroutines(arg.EvalScope.GoroutineID, arg.EvalScope.Frame, arg.EvalScope.DeferredCall, filter.Arg, arg.Start, arg.Count)
+			if len(gs) == arg.Count {
+				nextg = arg.Start + len(gs)
+			} else {
+				nextg = -1
+			}
+			gsLoaded = true
+			break
+		}
+	}
+
+	if !gsLoaded {
+		gs, nextg, err = s.debugger.Goroutines(arg.Start, arg.Count)
+	}
 	if err != nil {
 		return err
 	}
@@ -684,8 +718,8 @@ type AttachedToExistingProcessOut struct {
 }
 
 // AttachedToExistingProcess returns whether we attached to a running process or not
-func (c *RPCServer) AttachedToExistingProcess(arg AttachedToExistingProcessIn, out *AttachedToExistingProcessOut) error {
-	if c.config.Debugger.AttachPid != 0 {
+func (s *RPCServer) AttachedToExistingProcess(arg AttachedToExistingProcessIn, out *AttachedToExistingProcessOut) error {
+	if s.config.Debugger.AttachPid != 0 {
 		out.Answer = true
 	}
 	return nil
@@ -705,7 +739,8 @@ type FindLocationIn struct {
 }
 
 type FindLocationOut struct {
-	Locations []api.Location
+	Locations         []api.Location
+	SubstituteLocExpr string // if this isn't an empty string it should be passed as the location expression for CreateBreakpoint instead of the original location expression
 }
 
 // FindLocation returns concrete location information described by a location expression.
@@ -721,9 +756,9 @@ type FindLocationOut struct {
 //	* *<address> returns the location corresponding to the specified address
 //
 // NOTE: this function does not actually set breakpoints.
-func (c *RPCServer) FindLocation(arg FindLocationIn, out *FindLocationOut) error {
+func (s *RPCServer) FindLocation(arg FindLocationIn, out *FindLocationOut) error {
 	var err error
-	out.Locations, err = c.debugger.FindLocation(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Loc, arg.IncludeNonExecutableLines, arg.SubstitutePathRules)
+	out.Locations, out.SubstituteLocExpr, err = s.debugger.FindLocation(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Loc, arg.IncludeNonExecutableLines, arg.SubstitutePathRules)
 	return err
 }
 
@@ -744,15 +779,15 @@ type DisassembleOut struct {
 // Scope is used to mark the instruction the specified goroutine is stopped at.
 //
 // Disassemble will also try to calculate the destination address of an absolute indirect CALL if it happens to be the instruction the selected goroutine is stopped at.
-func (c *RPCServer) Disassemble(arg DisassembleIn, out *DisassembleOut) error {
+func (s *RPCServer) Disassemble(arg DisassembleIn, out *DisassembleOut) error {
 	var err error
-	insts, err := c.debugger.Disassemble(arg.Scope.GoroutineID, arg.StartPC, arg.EndPC)
+	insts, err := s.debugger.Disassemble(arg.Scope.GoroutineID, arg.StartPC, arg.EndPC)
 	if err != nil {
 		return err
 	}
 	out.Disassemble = make(api.AsmInstructions, len(insts))
 	for i := range insts {
-		out.Disassemble[i] = api.ConvertAsmInstruction(insts[i], c.debugger.AsmInstructionText(&insts[i], proc.AssemblyFlavour(arg.Flavour)))
+		out.Disassemble[i] = api.ConvertAsmInstruction(insts[i], s.debugger.AsmInstructionText(&insts[i], proc.AssemblyFlavour(arg.Flavour)))
 	}
 	return nil
 }
@@ -878,12 +913,13 @@ func (s *RPCServer) ListDynamicLibraries(in ListDynamicLibrariesIn, out *ListDyn
 	return nil
 }
 
-// ListPackagesBuildInfoIn holds the arguments of ListPackages.
+// ListPackagesBuildInfoIn holds the arguments of ListPackagesBuildInfo.
 type ListPackagesBuildInfoIn struct {
 	IncludeFiles bool
+	Filter       string // if not empty, returns only packages matching the regexp.
 }
 
-// ListPackagesBuildInfoOut holds the return values of ListPackages.
+// ListPackagesBuildInfoOut holds the return values of ListPackagesBuildInfo.
 type ListPackagesBuildInfoOut struct {
 	List []api.PackageBuildInfo
 }
@@ -894,9 +930,20 @@ type ListPackagesBuildInfoOut struct {
 // Note that the directory path is a best guess and may be wrong is a tool
 // other than cmd/go is used to perform the build.
 func (s *RPCServer) ListPackagesBuildInfo(in ListPackagesBuildInfoIn, out *ListPackagesBuildInfoOut) error {
+	var pattern *regexp.Regexp
+	if in.Filter != "" {
+		p, err := regexp.Compile(in.Filter)
+		if err != nil {
+			return fmt.Errorf("invalid Filter pattern: %v", err)
+		}
+		pattern = p
+	}
 	pkgs := s.debugger.ListPackagesBuildInfo(in.IncludeFiles)
 	out.List = make([]api.PackageBuildInfo, 0, len(pkgs))
 	for _, pkg := range pkgs {
+		if pattern != nil && !pattern.MatchString(pkg.ImportPath) {
+			continue
+		}
 		var files []string
 
 		if len(pkg.Files) > 0 {
