@@ -3,7 +3,6 @@ package service_test
 import (
 	"flag"
 	"fmt"
-	"io/ioutil"
 	"math/rand"
 	"net"
 	"net/rpc"
@@ -85,7 +84,9 @@ func startServer(name string, buildFlags protest.BuildFlags, t *testing.T, redir
 			Packages:       []string{fixture.Source},
 			BuildFlags:     "", // build flags can be an empty string here because the only test that uses it, does not set special flags.
 			ExecuteKind:    debugger.ExecutingGeneratedFile,
-			Redirects:      redirects,
+			Stdin:          redirects[0],
+			Stdout:         proc.OutputRedirect{Path: redirects[1]},
+			Stderr:         proc.OutputRedirect{Path: redirects[2]},
 		},
 	})
 	if err := server.Run(); err != nil {
@@ -229,7 +230,7 @@ func main() {
 
 func TestRestart_rebuild(t *testing.T) {
 	// In the original fixture file the env var tested for is SOMEVAR.
-	os.Setenv("SOMEVAR", "bah")
+	t.Setenv("SOMEVAR", "bah")
 
 	withTestClient2Extended("testenv", t, 0, [3]string{}, nil, func(c service.Client, f protest.Fixture) {
 		<-c.Continue()
@@ -244,19 +245,19 @@ func TestRestart_rebuild(t *testing.T) {
 		fi, err := os.Stat(f.Source)
 		assertNoError(err, t, "Stat fixture.Source")
 
-		originalSource, err := ioutil.ReadFile(f.Source)
+		originalSource, err := os.ReadFile(f.Source)
 		assertNoError(err, t, "Reading original source")
 
 		// Ensure we write the original source code back after the test exits.
-		defer ioutil.WriteFile(f.Source, originalSource, fi.Mode())
+		defer os.WriteFile(f.Source, originalSource, fi.Mode())
 
 		// Write modified source code to the fixture file.
-		err = ioutil.WriteFile(f.Source, []byte(modifiedSource), fi.Mode())
+		err = os.WriteFile(f.Source, []byte(modifiedSource), fi.Mode())
 		assertNoError(err, t, "Writing modified source")
 
 		// First set our new env var and ensure later that the
 		// modified source code picks it up.
-		os.Setenv("SOMEMODIFIEDVAR", "foobar")
+		t.Setenv("SOMEMODIFIEDVAR", "foobar")
 
 		// Restart the program, rebuilding from source.
 		_, err = c.Restart(true)
@@ -863,7 +864,7 @@ func TestClientServer_traceContinue(t *testing.T) {
 					t.Fatalf("No goroutine information")
 				}
 
-				if len(bpi.Stacktrace) <= 0 {
+				if len(bpi.Stacktrace) == 0 {
 					t.Fatalf("No stacktrace\n")
 				}
 
@@ -944,6 +945,9 @@ func TestClientServer_traceContinue2(t *testing.T) {
 }
 
 func TestClientServer_FindLocations(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" && buildMode == "pie" {
+		t.Skip("pie mode broken on ppc64le")
+	}
 	withTestClient2("locationsprog", t, func(c service.Client) {
 		someFunctionCallAddr := findLocationHelper(t, c, "locationsprog.go:26", false, 1, 0)[0]
 		someFunctionLine1 := findLocationHelper(t, c, "locationsprog.go:27", false, 1, 0)[0]
@@ -992,14 +996,14 @@ func TestClientServer_FindLocations(t *testing.T) {
 
 		findLocationHelper(t, c, `*amap["k"]`, false, 1, findLocationHelper(t, c, `amap["k"]`, false, 1, 0)[0])
 
-		locsNoSubst, _ := c.FindLocation(api.EvalScope{GoroutineID: -1}, "_fixtures/locationsprog.go:35", false, nil)
+		locsNoSubst, _, _ := c.FindLocation(api.EvalScope{GoroutineID: -1}, "_fixtures/locationsprog.go:35", false, nil)
 		sep := "/"
 		if strings.Contains(locsNoSubst[0].File, "\\") {
 			sep = "\\"
 		}
 		substRules := [][2]string{{strings.Replace(locsNoSubst[0].File, "locationsprog.go", "", 1), strings.Replace(locsNoSubst[0].File, "_fixtures"+sep+"locationsprog.go", "nonexistent", 1)}}
 		t.Logf("substitute rules: %q -> %q", substRules[0][0], substRules[0][1])
-		locsSubst, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "nonexistent/locationsprog.go:35", false, substRules)
+		locsSubst, _, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "nonexistent/locationsprog.go:35", false, substRules)
 		if err != nil {
 			t.Fatalf("FindLocation(locationsprog.go:35) with substitute rules: %v", err)
 		}
@@ -1109,7 +1113,7 @@ func TestClientServer_FindLocations(t *testing.T) {
 }
 
 func findLocationHelper2(t *testing.T, c service.Client, loc string, checkLoc *api.Location) *api.Location {
-	locs, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, loc, false, nil)
+	locs, _, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, loc, false, nil)
 	if err != nil {
 		t.Fatalf("FindLocation(%q) -> error %v", loc, err)
 	}
@@ -1208,6 +1212,9 @@ func TestClientServer_FullStacktrace(t *testing.T) {
 	if runtime.GOOS == "darwin" && runtime.GOARCH == "arm64" {
 		t.Skip("cgo doesn't work on darwin/arm64")
 	}
+	if runtime.GOARCH == "ppc64le" && buildMode == "pie" {
+		t.Skip("pie mode broken on ppc64le")
+	}
 
 	lenient := false
 	if runtime.GOOS == "windows" {
@@ -1352,7 +1359,7 @@ func TestIssue355(t *testing.T) {
 		assertError(err, t, "ListGoroutines()")
 		_, err = c.Stacktrace(gid, 10, 0, &normalLoadConfig)
 		assertError(err, t, "Stacktrace()")
-		_, err = c.FindLocation(api.EvalScope{GoroutineID: gid}, "+1", false, nil)
+		_, _, err = c.FindLocation(api.EvalScope{GoroutineID: gid}, "+1", false, nil)
 		assertError(err, t, "FindLocation()")
 		_, err = c.DisassemblePC(api.EvalScope{GoroutineID: -1}, 0x40100, api.IntelFlavour)
 		assertError(err, t, "DisassemblePC()")
@@ -1360,6 +1367,9 @@ func TestIssue355(t *testing.T) {
 }
 
 func TestDisasm(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" {
+		t.Skip("skipped on ppc64le: broken")
+	}
 	// Tests that disassembling by PC, range, and current PC all yield similar results
 	// Tests that disassembly by current PC will return a disassembly containing the instruction at PC
 	// Tests that stepping on a calculated CALL instruction will yield a disassembly that contains the
@@ -1369,7 +1379,7 @@ func TestDisasm(t *testing.T) {
 		state := <-ch
 		assertNoError(state.Err, t, "Continue()")
 
-		locs, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "main.main", false, nil)
+		locs, _, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "main.main", false, nil)
 		assertNoError(err, t, "FindLocation()")
 		if len(locs) != 1 {
 			t.Fatalf("wrong number of locations for main.main: %d", len(locs))
@@ -1533,7 +1543,7 @@ func TestSkipPrologue(t *testing.T) {
 		findLocationHelper(t, c, "*fn1", false, 1, afunction)
 		findLocationHelper(t, c, "locationsprog2.go:8", false, 1, afunction)
 
-		afunction0 := uint64(clientEvalVariable(t, c, "main.afunction").Addr)
+		afunction0 := clientEvalVariable(t, c, "main.afunction").Addr
 
 		if afunction == afunction0 {
 			t.Fatal("Skip prologue failed")
@@ -1544,21 +1554,21 @@ func TestSkipPrologue(t *testing.T) {
 func TestSkipPrologue2(t *testing.T) {
 	withTestClient2("callme", t, func(c service.Client) {
 		callme := findLocationHelper(t, c, "main.callme", false, 1, 0)[0]
-		callmeZ := uint64(clientEvalVariable(t, c, "main.callme").Addr)
+		callmeZ := clientEvalVariable(t, c, "main.callme").Addr
 		findLocationHelper(t, c, "callme.go:5", false, 1, callme)
 		if callme == callmeZ {
 			t.Fatal("Skip prologue failed")
 		}
 
 		callme2 := findLocationHelper(t, c, "main.callme2", false, 1, 0)[0]
-		callme2Z := uint64(clientEvalVariable(t, c, "main.callme2").Addr)
+		callme2Z := clientEvalVariable(t, c, "main.callme2").Addr
 		findLocationHelper(t, c, "callme.go:12", false, 1, callme2)
 		if callme2 == callme2Z {
 			t.Fatal("Skip prologue failed")
 		}
 
 		callme3 := findLocationHelper(t, c, "main.callme3", false, 1, 0)[0]
-		callme3Z := uint64(clientEvalVariable(t, c, "main.callme3").Addr)
+		callme3Z := clientEvalVariable(t, c, "main.callme3").Addr
 		ver, _ := goversion.Parse(runtime.Version())
 
 		if (ver.Major < 0 || ver.AfterOrEqual(goversion.GoVer18Beta)) && runtime.GOARCH != "386" {
@@ -1628,7 +1638,7 @@ func TestTypesCommand(t *testing.T) {
 func TestIssue406(t *testing.T) {
 	protest.AllowRecording(t)
 	withTestClient2("issue406", t, func(c service.Client) {
-		locs, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "issue406.go:146", false, nil)
+		locs, _, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "issue406.go:146", false, nil)
 		assertNoError(err, t, "FindLocation()")
 		_, err = c.CreateBreakpoint(&api.Breakpoint{Addr: locs[0].PC})
 		assertNoError(err, t, "CreateBreakpoint()")
@@ -2098,6 +2108,10 @@ func TestForceStopWhileContinue(t *testing.T) {
 }
 
 func TestClientServerFunctionCall(t *testing.T) {
+	if buildMode == "pie" && runtime.GOARCH == "ppc64le" {
+		t.Skip("Debug function call Test broken in PIE mode")
+	}
+
 	protest.MustSupportFunctionCalls(t, testBackend)
 	withTestClient2("fncall", t, func(c service.Client) {
 		c.SetReturnValuesLoadConfig(&normalLoadConfig)
@@ -2128,6 +2142,9 @@ func TestClientServerFunctionCall(t *testing.T) {
 }
 
 func TestClientServerFunctionCallPanic(t *testing.T) {
+	if buildMode == "pie" && runtime.GOARCH == "ppc64le" {
+		t.Skip("Debug function call Test broken in PIE mode")
+	}
 	protest.MustSupportFunctionCalls(t, testBackend)
 	withTestClient2("fncall", t, func(c service.Client) {
 		c.SetReturnValuesLoadConfig(&normalLoadConfig)
@@ -2183,9 +2200,7 @@ func TestAncestors(t *testing.T) {
 	if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 11) {
 		t.Skip("not supported on Go <= 1.10")
 	}
-	savedGodebug := os.Getenv("GODEBUG")
-	os.Setenv("GODEBUG", "tracebackancestors=100")
-	defer os.Setenv("GODEBUG", savedGodebug)
+	t.Setenv("GODEBUG", "tracebackancestors=100")
 	withTestClient2("testnextprog", t, func(c service.Client) {
 		_, err := c.CreateBreakpoint(&api.Breakpoint{FunctionName: "main.testgoroutine", Line: -1})
 		assertNoError(err, t, "CreateBreakpoint")
@@ -2242,7 +2257,7 @@ func TestUnknownMethodCall(t *testing.T) {
 func TestIssue1703(t *testing.T) {
 	// Calling Disassemble when there is no current goroutine should work.
 	withTestClient2("testnextprog", t, func(c service.Client) {
-		locs, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "main.main", true, nil)
+		locs, _, err := c.FindLocation(api.EvalScope{GoroutineID: -1}, "main.main", true, nil)
 		assertNoError(err, t, "FindLocation")
 		t.Logf("FindLocation: %#v", locs)
 		text, err := c.DisassemblePC(api.EvalScope{GoroutineID: -1}, locs[0].PC, api.IntelFlavour)
@@ -2400,7 +2415,7 @@ func TestRedirects(t *testing.T) {
 	withTestClient2Extended("redirect", t, 0, [3]string{infile, outfile, ""}, nil, func(c service.Client, fixture protest.Fixture) {
 		outpath := filepath.Join(fixture.BuildDir, outfile)
 		<-c.Continue()
-		buf, err := ioutil.ReadFile(outpath)
+		buf, err := os.ReadFile(outpath)
 		assertNoError(err, t, "Reading output file")
 		t.Logf("output %q", buf)
 		if !strings.HasPrefix(string(buf), "Redirect test") {
@@ -2411,7 +2426,7 @@ func TestRedirects(t *testing.T) {
 			_, err = c.Restart(false)
 			assertNoError(err, t, "Restart")
 			<-c.Continue()
-			buf2, err := ioutil.ReadFile(outpath)
+			buf2, err := os.ReadFile(outpath)
 			t.Logf("output %q", buf2)
 			assertNoError(err, t, "Reading output file (second time)")
 			if !strings.HasPrefix(string(buf2), "Redirect test") {
@@ -2566,7 +2581,7 @@ func TestGoroutinesGrouping(t *testing.T) {
 	withTestClient2("goroutinegroup", t, func(c service.Client) {
 		state := <-c.Continue()
 		assertNoError(state.Err, t, "Continue")
-		_, ggrp, _, _, err := c.ListGoroutinesWithFilter(0, 0, nil, &api.GoroutineGroupingOptions{GroupBy: api.GoroutineLabel, GroupByKey: "name", MaxGroupMembers: 5, MaxGroups: 10})
+		_, ggrp, _, _, err := c.ListGoroutinesWithFilter(0, 0, nil, &api.GoroutineGroupingOptions{GroupBy: api.GoroutineLabel, GroupByKey: "name", MaxGroupMembers: 5, MaxGroups: 10}, nil)
 		assertNoError(err, t, "ListGoroutinesWithFilter (group by label)")
 		t.Logf("%#v\n", ggrp)
 		if len(ggrp) < 5 {
@@ -2579,7 +2594,7 @@ func TestGoroutinesGrouping(t *testing.T) {
 				break
 			}
 		}
-		gs, _, _, _, err := c.ListGoroutinesWithFilter(0, 0, []api.ListGoroutinesFilter{{Kind: api.GoroutineLabel, Arg: "name="}}, nil)
+		gs, _, _, _, err := c.ListGoroutinesWithFilter(0, 0, []api.ListGoroutinesFilter{{Kind: api.GoroutineLabel, Arg: "name="}}, nil, nil)
 		assertNoError(err, t, "ListGoroutinesWithFilter (filter unnamed)")
 		if len(gs) != unnamedCount {
 			t.Errorf("wrong number of goroutines returned by filter: %d (expected %d)\n", len(gs), unnamedCount)
@@ -2774,6 +2789,9 @@ func TestNonGoDebug(t *testing.T) {
 	if runtime.GOOS != "linux" {
 		t.Skip()
 	}
+	if objcopyPath, _ := exec.LookPath("cc"); objcopyPath == "" {
+		t.Skip("no C compiler in path")
+	}
 	dir := protest.FindFixturesDir()
 	path := protest.TempFile("testc")
 	cmd := exec.Command("cc", "-g", "-o", path, filepath.Join(dir, "test.c"))
@@ -2813,12 +2831,13 @@ func TestRestart_PreserveFunctionBreakpoint(t *testing.T) {
 	// even if the function changed position in the source file.
 
 	dir := protest.FindFixturesDir()
+	outpath := filepath.Join(dir, "testfnpos.go")
+	defer os.Remove(outpath)
 
 	copy := func(inpath string) {
-		buf, err := ioutil.ReadFile(inpath)
+		buf, err := os.ReadFile(inpath)
 		assertNoError(err, t, fmt.Sprintf("Reading %q", inpath))
-		outpath := filepath.Join(dir, "testfnpos.go")
-		assertNoError(ioutil.WriteFile(outpath, buf, 0666), t, fmt.Sprintf("Creating %q", outpath))
+		assertNoError(os.WriteFile(outpath, buf, 0o666), t, fmt.Sprintf("Creating %q", outpath))
 	}
 
 	copy(filepath.Join(dir, "testfnpos1.go"))
@@ -2856,6 +2875,9 @@ func assertLine(t *testing.T, state *api.DebuggerState, file string, lineno int)
 }
 
 func TestPluginSuspendedBreakpoint(t *testing.T) {
+	if runtime.GOARCH == "ppc64le" {
+		t.Skip("skipped on ppc64le: broken")
+	}
 	// Tests that breakpoints created in a suspended state will be enabled automatically when a plugin is loaded.
 	pluginFixtures := protest.WithPlugins(t, protest.AllNonOptimized, "plugin1/", "plugin2/")
 	dir, err := filepath.Abs(protest.FindFixturesDir())
@@ -2973,7 +2995,7 @@ func TestClientServer_createBreakpointWithID(t *testing.T) {
 }
 
 func TestClientServer_autoBreakpoints(t *testing.T) {
-	// Check that unrecoverd-panic and fatal-throw breakpoints are visible in
+	// Check that unrecovered-panic and fatal-throw breakpoints are visible in
 	// the breakpoint list.
 	protest.AllowRecording(t)
 	withTestClient2("math", t, func(c service.Client) {
@@ -3017,3 +3039,69 @@ func TestClientServer_breakpointOnFuncWithABIWrapper(t *testing.T) {
 		}
 	})
 }
+
+var waitReasonStrings = [...]string{
+	"",
+	"GC assist marking",
+	"IO wait",
+	"chan receive (nil chan)",
+	"chan send (nil chan)",
+	"dumping heap",
+	"garbage collection",
+	"garbage collection scan",
+	"panicwait",
+	"select",
+	"select (no cases)",
+	"GC assist wait",
+	"GC sweep wait",
+	"GC scavenge wait",
+	"chan receive",
+	"chan send",
+	"finalizer wait",
+	"force gc (idle)",
+	"semacquire",
+	"sleep",
+	"sync.Cond.Wait",
+	"timer goroutine (idle)",
+	"trace reader (blocked)",
+	"wait for GC cycle",
+	"GC worker (idle)",
+	"preempted",
+	"debug call",
+}
+
+func TestClientServer_chanGoroutines(t *testing.T) {
+	withTestClient2("changoroutines", t, func(c service.Client) {
+		state := <-c.Continue()
+		assertNoError(state.Err, t, "Continue()")
+
+		countRecvSend := func(gs []*api.Goroutine) (recvq, sendq int) {
+			for _, g := range gs {
+				t.Logf("\tID: %d WaitReason: %s\n", g.ID, waitReasonStrings[g.WaitReason])
+				switch waitReasonStrings[g.WaitReason] {
+				case "chan send":
+					sendq++
+				case "chan receive":
+					recvq++
+				}
+			}
+			return
+		}
+
+		gs, _, _, _, err := c.ListGoroutinesWithFilter(0, 100, []api.ListGoroutinesFilter{{Kind: api.GoroutineWaitingOnChannel, Arg: "blockingchan1"}}, nil, &api.EvalScope{GoroutineID: -1})
+		assertNoError(err, t, "ListGoroutinesWithFilter(blockingchan1)")
+		t.Logf("blockingchan1 gs:")
+		recvq, sendq := countRecvSend(gs)
+		if len(gs) != 2 || recvq != 0 || sendq != 2 {
+			t.Error("wrong number of goroutines for blockingchan1")
+		}
+
+		gs, _, _, _, err = c.ListGoroutinesWithFilter(0, 100, []api.ListGoroutinesFilter{{Kind: api.GoroutineWaitingOnChannel, Arg: "blockingchan2"}}, nil, &api.EvalScope{GoroutineID: -1})
+		assertNoError(err, t, "ListGoroutinesWithFilter(blockingchan2)")
+		t.Logf("blockingchan2 gs:")
+		recvq, sendq = countRecvSend(gs)
+		if len(gs) != 1 || recvq != 1 || sendq != 0 {
+			t.Error("wrong number of goroutines for blockingchan2")
+		}
+	})
+}
